from msgbus import (
    TK_SYS_NAK,
    TK_EVT_EVENT,
)
from .. import logger
from .. import mb_context
from model.customexception import CleanRuptureException


def clean_rupture_on_open_day():
    # type: (...) -> None
    try:
        dest_name = "Ruptura"
        data = '\0CleanRupture\0OnOpenDay'
        msg = mb_context.MB_EasySendMessage(
            dest_name=dest_name,
            token=TK_EVT_EVENT,
            data=data,
        )
        if msg.token == TK_SYS_NAK:
            raise CleanRuptureException(msg.data)
    except (Exception, CleanRuptureException) as e:
        logger.exception('[clean_rupture] Exception {}'.format(e))



import os

from systools import sys_parsefilepath

from application.model import UpdateType
from application.model.customexception import ErrorUpdatingStartStop
from application.service import UpdaterService
from application.util import (
    LoggerUtil,
    settings,
)

GENESIS_PATH = "../../genesis"

logger = LoggerUtil.get_logger_name(__name__)


class StartStopUpdaterService(object):
    def __init__(
        self,
        updater_service,  # type: UpdaterService
    ):
        # type: (...) -> None

        self.update_type = UpdateType.binaries
        self.update_name = self.update_type.name
        data_path = sys_parsefilepath("{HVDATADIR}")
        self.genesis_path = os.path.join(
            data_path,
            GENESIS_PATH,
        )
        self.updater_service = updater_service
        self.core_relative_linux_path = "linux-x86_64"
        self.core_relative_windows_path = "windows-x86"

    def update_start_stop(self):
        # type: (...) -> None

        if self._check_isdev():
            return

        settings.set_update_control(
            self.update_name,
            "$UPDATING_START_STOP_FILES",
        )
        genesis_bin_path_linux = os.path.join(
            self.genesis_path,
            "bin",
            self.core_relative_linux_path,
        )
        genesis_bin_path_windows = os.path.join(
            self.genesis_path,
            "bin",
            self.core_relative_windows_path,
        )
        try:
            logger.info("Updating start/stop files")
            if os.path.exists(genesis_bin_path_linux) and os.path.exists(genesis_bin_path_windows):
                genesis_root_path_linux = os.path.join(
                    self.genesis_path,
                    "root",
                    self.core_relative_linux_path,
                )
                genesis_root_path_windows = os.path.join(
                    self.genesis_path,
                    "root",
                    self.core_relative_windows_path,
                )

                files_to_root_folder = ["start_node", "stop", "E-Deploy POS"]
                files_to_pos_folder = ["start_server", "stop", "E-Deploy POS"]
                files_to_remove = ["start_node", "start_server", "stop", "E-Deploy POS"]

                for destination_path in [genesis_root_path_linux, genesis_root_path_windows]:
                    self.updater_service.create_dir(
                        path=destination_path,
                        delete_if_exists=False
                    )
                    self._start_stop_copy_process(
                        files_to_copy=files_to_root_folder,
                        original_path=genesis_bin_path_linux if self.core_relative_linux_path in destination_path else genesis_bin_path_windows,
                        destination_path=destination_path,
                        file_to_rename="start_node",
                    )

                pos_path = os.path.join(
                    self.genesis_path,
                    "..",
                    "",
                )
                self._start_stop_copy_process(
                    files_to_copy=files_to_pos_folder,
                    original_path=genesis_bin_path_windows if self._is_windows() else genesis_bin_path_linux,
                    destination_path=pos_path,
                    file_to_rename="start_server",
                )

                for genesis_bin_path in [genesis_bin_path_linux, genesis_bin_path_windows]:
                    for file_name in files_to_remove:
                        self._remove_file(
                            file_name=file_name,
                            path=genesis_bin_path,
                        )

            logger.info("Update start/stop successfully!")
        except Exception as _:
            logger.exception("Error updating start/stop")
            raise ErrorUpdatingStartStop()

    def has_genesis_root_path(
        self
    ):
        # type: () -> bool

        if self._check_isdev():
            return True

        genesis_root_path = os.path.join(self.genesis_path, "root")
        return os.path.exists(genesis_root_path)

    def _start_stop_copy_process(
        self,
        files_to_copy,     # type: list
        original_path,     # type: str
        destination_path,  # type: str
        file_to_rename,    # type: str
    ):
        # type: (...) -> None

        for file_name in files_to_copy:
            original_file_path = os.path.join(
                original_path,
                "{0}.{1}".format(
                    file_name,
                    self._get_extension(
                        file_name,
                        original_path,
                    ),
                ),
            )
            if os.path.exists(original_file_path):
                destination_file_name = file_name if file_name != file_to_rename else "start"
                self._remove_file(
                    file_name=destination_file_name,
                    path=destination_path,
                )

                self.updater_service.copy_file(
                    original_file_path=original_file_path,
                    destination_file_path=os.path.join(
                        destination_path,
                        "{0}.{1}".format(
                            destination_file_name,
                            self._get_extension(
                                destination_file_name,
                                destination_path,
                            ),
                        ),
                    ),
                )

    def _remove_file(
        self,
        file_name,  # type: str
        path,       # type: str
    ):
        # type: (...) -> None

        file_path = os.path.join(
            path,
            "{0}.{1}".format(
                file_name,
                self._get_extension(
                    file_name,
                    path,
                ),
            ),
        )
        if os.path.exists(file_path):
            os.remove(file_path)

    def _get_extension(
        self,
        file_name,  # type: str
        path=None,  # type: str
    ):
        # type: (...) -> str

        linux_extension = "sh"
        win_extension = "bat" if file_name != "E-Deploy POS" else "exe"
        so_extension = win_extension if self._is_windows() else linux_extension
        if path is None:
            return so_extension
        if self.core_relative_windows_path in path:
            return win_extension
        if self.core_relative_linux_path in path:
            return linux_extension
        return so_extension

    @staticmethod
    def _is_windows():
        # type: () -> bool

        return "win" in os.sys.platform

    @staticmethod
    def _check_isdev(
    ):
        # type: (...) -> bool

        return os.environ.get("DEBUG", "").upper() == "TRUE"

    @staticmethod
    def _search_in_file(
        file_path,             # type: str
        target,                # type: str
        line_scan_limit=None,  # type: int
    ):
        # type: (...) -> bool

        try:
            with open(file_path, 'r+') as f:
                scanned_line_counter = 0
                for line in f:
                    line = line.strip()
                    if target in line:
                        return True
                    if line_scan_limit is not None and scanned_line_counter >= line_scan_limit:
                        return False
                    scanned_line_counter += 1
        except Exception:
            return False
        return False



from production.box import AddTimeTagBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class ConfigDoesNotExist(Exception):
    def __init__(self, config_name):
        self.message = "({}) does not exist".format(config_name)


class AddTimeTagBoxBuilder(BoxBuilder):
    def __init__(self, loggers):
        super(AddTimeTagBoxBuilder, self).__init__(loggers)

    def build(self, config):
        # type: (BoxConfiguration) -> AddTimeTagBox
        def get_config(config_name):
            return self.get_extra_config(config, config_name)

        return AddTimeTagBox(config.name, config.sons, self.get_logger(config), get_config)

    def get_extra_config(self, config, config_name):
        config_value = self.get_extra(config, config_name, None)
        if config_value is None:
            self.loggers['DEBUG'].info("({}) does not exist".format(config_name))
            raise ConfigDoesNotExist(config_name)

        return config_value



def migrate(conn, tables):
    import os
    from systools import sys_log_debug
    updateTablesPriority = ["Users"]

    sys_log_debug("[migrate] start")
    for table in updateTablesPriority:
        if tables["Users"]:
            sys_log_debug("[migrate] table {}".format(table))
            strCommonFields = ", ".join(tables[table]["fields_Common"])
            sys_log_debug("[migrate] copying data for table {}".format(table))
            if "NfcUuid" not in tables[table]["fields_Common"]:
                newFields = strCommonFields + ", NfcUuid"
                oldFields = strCommonFields + ", Password"
                stmt = "INSERT OR REPLACE INTO %s (%s) SELECT %s FROM old.%s;" % (table, newFields, oldFields, table)
            else:
                stmt = "INSERT OR REPLACE INTO %s (%s) SELECT %s FROM old.%s;" % (table, strCommonFields, strCommonFields, table)
            conn.query(stmt)
            sys_log_debug("[migrate] finished query for table {}".format(table))
        else:
            sys_log_debug("[migrate] table {}".format(table))
            strCommonFields = ", ".join(tables[table]["fields_Common"])
            sys_log_debug("[migrate] copying data for table {}".format(table))
            stmt = "INSERT OR REPLACE INTO %s (%s) SELECT %s FROM old.%s;" % (table, strCommonFields, strCommonFields, table)
            conn.query(stmt)
            sys_log_debug("[migrate] finished query for table {}".format(table))

    sys_log_debug("[migrate] finished iterating")



# -*- coding: utf-8 -*-

import logging

from sysactions import action, show_confirmation, show_messagebox, StopAction
from bustoken import (
    TK_REMOTE_ORDER_SEND_ORDER_TO_LOGISTIC,
    TK_REMOTE_ORDER_SEND_DELIVERY_MANUAL_ORDER_AUTOMATIC_LOGISTIC,
)
from msgbus import TK_SYS_ACK

from .. import mb_context

logger = logging.getLogger("PosActions")

@action
def send_logistic_remote_order(pos_id, order_id=""):
    confirmation = show_confirmation(pos_id, "$LOGISTIC_SEND_CONFIRMATION")
    if not confirmation:
        raise StopAction()

    data = pos_id + "\0" + order_id
    logistic_send_order_token = TK_REMOTE_ORDER_SEND_ORDER_TO_LOGISTIC
    msg = mb_context.MB_EasySendMessage("RemoteOrder", logistic_send_order_token, data=data, timeout=10000000)
    if msg.token == TK_SYS_ACK:
        show_messagebox(pos_id, "$LOGISTIC_SENT")
        return

    show_messagebox(pos_id, "$LOGISTIC_SENT_ERROR")


def send_delivery_manual_order_automatic_logistic(
    order_id,     # type: str
):
    # type: (...) -> None

    try:
        response = mb_context.MB_EasySendMessage(
            dest_name="RemoteOrder",
            token=TK_REMOTE_ORDER_SEND_DELIVERY_MANUAL_ORDER_AUTOMATIC_LOGISTIC,
            data=order_id,
            timeout=10000000,
        )

        if response.token != TK_SYS_ACK:
            logger.error("NAK send delivery manual order automatic logistic: {}".format(response.data))

    except Exception as _:
        logger.exception("Error send delivery manual order automatic logistic")




from messagebus import (
    AckMessage,
    Event,
    Message,
    MessageBus,
    NakMessage
)
from typing import Any

from ._CommonProcessor import _CommonProcessor


class GetNameProcessor(_CommonProcessor):
    def parse_data(
        self,
        data,           # type: str
        event_type,     # type: str
    ):
        # type: (...) -> str

        return data

    def call_business(
        self,
        pos_id,     # type: str
    ):
        # type: (...) -> str

        return self.payment_router.get_name()

    def format_response(
        self,
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Event
        input_obj,      # type: Any
        result,         # type: Any
    ):
        # type: (...) -> None

        message_bus.reply_message(
            message=message,
            reply_message=AckMessage(data=result),
        )

    def format_exception(
        self,
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Event
        input_obj,      # type: Any
        exception,      # type: Exception
    ):
        # type: (...) -> None

        self.payment_router._logger.exception("exception: {}".format(exception))
        message_bus.reply_message(
            message=message,
            reply_message=NakMessage(data=exception.message),
        )



from threading import Lock

from systools import sys_log_error
from typing import Dict

user_lock = {}  # type: Dict[str, Lock]
management_lock = Lock()


def acquire_user_lock(
    user_id,    # type: str
):
    # type: (...) -> bool

    if user_id not in user_lock:
        user_lock[user_id] = Lock()

    with management_lock:
        already_locked = user_lock[user_id].locked()
        if already_locked:
            return False

        user_lock[user_id].acquire()
        return True


def release_user_lock(
    user_id,    # type: str
):
    # type: (...) -> None

    with management_lock:
        if user_id not in user_lock or not user_lock[user_id].locked():
            sys_log_error("User lock not found for user_id: {}".format(user_id))
            return
        user_lock[user_id].release()



#!/usr/bin/env python
#
# backend for serial IO for POSIX compatible systems, like Linux, OSX
#
# This file is part of pySerial. https://github.com/pyserial/pyserial
# (C) 2001-2020 Chris Liechti <cliechti@gmx.net>
#
# SPDX-License-Identifier:    BSD-3-Clause
#
# parts based on code from Grant B. Edwards  <grante@visi.com>:
#  ftp://ftp.visi.com/users/grante/python/PosixSerial.py
#
# references: http://www.easysw.com/~mike/serial/serial.html

# Collection of port names (was previously used by number_to_device which was
# removed.
# - Linux                   /dev/ttyS%d (confirmed)
# - cygwin/win32            /dev/com%d (confirmed)
# - openbsd (OpenBSD)       /dev/cua%02d
# - bsd*, freebsd*          /dev/cuad%d
# - darwin (OS X)           /dev/cuad%d
# - netbsd                  /dev/dty%02d (NetBSD 1.6 testing by Erk)
# - irix (IRIX)             /dev/ttyf%d (partially tested) names depending on flow control
# - hp (HP-UX)              /dev/tty%dp0 (not tested)
# - sunos (Solaris/SunOS)   /dev/tty%c (letters, 'a'..'z') (confirmed)
# - aix (AIX)               /dev/tty%d


from __future__ import absolute_import

# pylint: disable=abstract-method
import errno
import fcntl
import os
import select
import struct
import sys
import termios

import serial
from serial.serialutil import SerialBase, SerialException, to_bytes, \
    PortNotOpenError, SerialTimeoutException, Timeout


class PlatformSpecificBase(object):
    BAUDRATE_CONSTANTS = {}

    def _set_special_baudrate(self, baudrate):
        raise NotImplementedError('non-standard baudrates are not supported on this platform')

    def _set_rs485_mode(self, rs485_settings):
        raise NotImplementedError('RS485 not supported on this platform')

    def set_low_latency_mode(self, low_latency_settings):
        raise NotImplementedError('Low latency not supported on this platform')

    def _update_break_state(self):
        """\
        Set break: Controls TXD. When active, no transmitting is possible.
        """
        if self._break_state:
            fcntl.ioctl(self.fd, TIOCSBRK)
        else:
            fcntl.ioctl(self.fd, TIOCCBRK)
    

# some systems support an extra flag to enable the two in POSIX unsupported
# paritiy settings for MARK and SPACE
CMSPAR = 0  # default, for unsupported platforms, override below

# try to detect the OS so that a device can be selected...
# this code block should supply a device() and set_special_baudrate() function
# for the platform
plat = sys.platform.lower()

if plat[:5] == 'linux':    # Linux (confirmed)  # noqa
    import array

    # extra termios flags
    CMSPAR = 0o10000000000  # Use "stick" (mark/space) parity

    # baudrate ioctls
    TCGETS2 = 0x802C542A
    TCSETS2 = 0x402C542B
    BOTHER = 0o010000

    # RS485 ioctls
    TIOCGRS485 = 0x542E
    TIOCSRS485 = 0x542F
    SER_RS485_ENABLED = 0b00000001
    SER_RS485_RTS_ON_SEND = 0b00000010
    SER_RS485_RTS_AFTER_SEND = 0b00000100
    SER_RS485_RX_DURING_TX = 0b00010000

    class PlatformSpecific(PlatformSpecificBase):
        BAUDRATE_CONSTANTS = {
            0:       0o000000,  # hang up
            50:      0o000001,
            75:      0o000002,
            110:     0o000003,
            134:     0o000004,
            150:     0o000005,
            200:     0o000006,
            300:     0o000007,
            600:     0o000010,
            1200:    0o000011,
            1800:    0o000012,
            2400:    0o000013,
            4800:    0o000014,
            9600:    0o000015,
            19200:   0o000016,
            38400:   0o000017,
            57600:   0o010001,
            115200:  0o010002,
            230400:  0o010003,
            460800:  0o010004,
            500000:  0o010005,
            576000:  0o010006,
            921600:  0o010007,
            1000000: 0o010010,
            1152000: 0o010011,
            1500000: 0o010012,
            2000000: 0o010013,
            2500000: 0o010014,
            3000000: 0o010015,
            3500000: 0o010016,
            4000000: 0o010017
        }

        def set_low_latency_mode(self, low_latency_settings):
            buf = array.array('i', [0] * 32)

            try:
                # get serial_struct
                fcntl.ioctl(self.fd, termios.TIOCGSERIAL, buf)

                # set or unset ASYNC_LOW_LATENCY flag
                if low_latency_settings:
                    buf[4] |= 0x2000
                else:
                    buf[4] &= ~0x2000

                # set serial_struct
                fcntl.ioctl(self.fd, termios.TIOCSSERIAL, buf)
            except IOError as e:
                raise ValueError('Failed to update ASYNC_LOW_LATENCY flag to {}: {}'.format(low_latency_settings, e))

        def _set_special_baudrate(self, baudrate):
            # right size is 44 on x86_64, allow for some growth
            buf = array.array('i', [0] * 64)
            try:
                # get serial_struct
                fcntl.ioctl(self.fd, TCGETS2, buf)
                # set custom speed
                buf[2] &= ~termios.CBAUD
                buf[2] |= BOTHER
                buf[9] = buf[10] = baudrate

                # set serial_struct
                fcntl.ioctl(self.fd, TCSETS2, buf)
            except IOError as e:
                raise ValueError('Failed to set custom baud rate ({}): {}'.format(baudrate, e))

        def _set_rs485_mode(self, rs485_settings):
            buf = array.array('i', [0] * 8)  # flags, delaytx, delayrx, padding
            try:
                fcntl.ioctl(self.fd, TIOCGRS485, buf)
                buf[0] |= SER_RS485_ENABLED
                if rs485_settings is not None:
                    if rs485_settings.loopback:
                        buf[0] |= SER_RS485_RX_DURING_TX
                    else:
                        buf[0] &= ~SER_RS485_RX_DURING_TX
                    if rs485_settings.rts_level_for_tx:
                        buf[0] |= SER_RS485_RTS_ON_SEND
                    else:
                        buf[0] &= ~SER_RS485_RTS_ON_SEND
                    if rs485_settings.rts_level_for_rx:
                        buf[0] |= SER_RS485_RTS_AFTER_SEND
                    else:
                        buf[0] &= ~SER_RS485_RTS_AFTER_SEND
                    if rs485_settings.delay_before_tx is not None:
                        buf[1] = int(rs485_settings.delay_before_tx * 1000)
                    if rs485_settings.delay_before_rx is not None:
                        buf[2] = int(rs485_settings.delay_before_rx * 1000)
                else:
                    buf[0] = 0  # clear SER_RS485_ENABLED
                fcntl.ioctl(self.fd, TIOCSRS485, buf)
            except IOError as e:
                raise ValueError('Failed to set RS485 mode: {}'.format(e))


elif plat == 'cygwin':       # cygwin/win32 (confirmed)

    class PlatformSpecific(PlatformSpecificBase):
        BAUDRATE_CONSTANTS = {
            128000: 0x01003,
            256000: 0x01005,
            500000: 0x01007,
            576000: 0x01008,
            921600: 0x01009,
            1000000: 0x0100a,
            1152000: 0x0100b,
            1500000: 0x0100c,
            2000000: 0x0100d,
            2500000: 0x0100e,
            3000000: 0x0100f
        }


elif plat[:6] == 'darwin':   # OS X
    import array
    IOSSIOSPEED = 0x80045402  # _IOW('T', 2, speed_t)

    class PlatformSpecific(PlatformSpecificBase):
        osx_version = os.uname()[2].split('.')
        TIOCSBRK = 0x2000747B # _IO('t', 123)
        TIOCCBRK = 0x2000747A # _IO('t', 122)

        # Tiger or above can support arbitrary serial speeds
        if int(osx_version[0]) >= 8:
            def _set_special_baudrate(self, baudrate):
                # use IOKit-specific call to set up high speeds
                buf = array.array('i', [baudrate])
                fcntl.ioctl(self.fd, IOSSIOSPEED, buf, 1)

        def _update_break_state(self):
            """\
            Set break: Controls TXD. When active, no transmitting is possible.
            """
            if self._break_state:
                fcntl.ioctl(self.fd, PlatformSpecific.TIOCSBRK)
            else:
                fcntl.ioctl(self.fd, PlatformSpecific.TIOCCBRK)

elif plat[:3] == 'bsd' or \
     plat[:7] == 'freebsd' or \
     plat[:6] == 'netbsd' or \
     plat[:7] == 'openbsd':

    class ReturnBaudrate(object):
        def __getitem__(self, key):
            return key

    class PlatformSpecific(PlatformSpecificBase):
        # Only tested on FreeBSD:
        # The baud rate may be passed in as
        # a literal value.
        BAUDRATE_CONSTANTS = ReturnBaudrate()

        TIOCSBRK = 0x2000747B # _IO('t', 123)
        TIOCCBRK = 0x2000747A # _IO('t', 122)

        
        def _update_break_state(self):
            """\
            Set break: Controls TXD. When active, no transmitting is possible.
            """
            if self._break_state:
                fcntl.ioctl(self.fd, PlatformSpecific.TIOCSBRK)
            else:
                fcntl.ioctl(self.fd, PlatformSpecific.TIOCCBRK)

else:
    class PlatformSpecific(PlatformSpecificBase):
        pass


# load some constants for later use.
# try to use values from termios, use defaults from linux otherwise
TIOCMGET = getattr(termios, 'TIOCMGET', 0x5415)
TIOCMBIS = getattr(termios, 'TIOCMBIS', 0x5416)
TIOCMBIC = getattr(termios, 'TIOCMBIC', 0x5417)
TIOCMSET = getattr(termios, 'TIOCMSET', 0x5418)

# TIOCM_LE = getattr(termios, 'TIOCM_LE', 0x001)
TIOCM_DTR = getattr(termios, 'TIOCM_DTR', 0x002)
TIOCM_RTS = getattr(termios, 'TIOCM_RTS', 0x004)
# TIOCM_ST = getattr(termios, 'TIOCM_ST', 0x008)
# TIOCM_SR = getattr(termios, 'TIOCM_SR', 0x010)

TIOCM_CTS = getattr(termios, 'TIOCM_CTS', 0x020)
TIOCM_CAR = getattr(termios, 'TIOCM_CAR', 0x040)
TIOCM_RNG = getattr(termios, 'TIOCM_RNG', 0x080)
TIOCM_DSR = getattr(termios, 'TIOCM_DSR', 0x100)
TIOCM_CD = getattr(termios, 'TIOCM_CD', TIOCM_CAR)
TIOCM_RI = getattr(termios, 'TIOCM_RI', TIOCM_RNG)
# TIOCM_OUT1 = getattr(termios, 'TIOCM_OUT1', 0x2000)
# TIOCM_OUT2 = getattr(termios, 'TIOCM_OUT2', 0x4000)
if hasattr(termios, 'TIOCINQ'):
    TIOCINQ = termios.TIOCINQ
else:
    TIOCINQ = getattr(termios, 'FIONREAD', 0x541B)
TIOCOUTQ = getattr(termios, 'TIOCOUTQ', 0x5411)

TIOCM_zero_str = struct.pack('I', 0)
TIOCM_RTS_str = struct.pack('I', TIOCM_RTS)
TIOCM_DTR_str = struct.pack('I', TIOCM_DTR)

TIOCSBRK = getattr(termios, 'TIOCSBRK', 0x5427)
TIOCCBRK = getattr(termios, 'TIOCCBRK', 0x5428)


class Serial(SerialBase, PlatformSpecific):
    """\
    Serial port class POSIX implementation. Serial port configuration is
    done with termios and fcntl. Runs on Linux and many other Un*x like
    systems.
    """

    def open(self):
        """\
        Open port with current settings. This may throw a SerialException
        if the port cannot be opened."""
        if self._port is None:
            raise SerialException("Port must be configured before it can be used.")
        if self.is_open:
            raise SerialException("Port is already open.")
        self.fd = None
        # open
        try:
            self.fd = os.open(self.portstr, os.O_RDWR | os.O_NOCTTY | os.O_NONBLOCK)
        except OSError as msg:
            self.fd = None
            raise SerialException(msg.errno, "could not open port {}: {}".format(self._port, msg))
        #~ fcntl.fcntl(self.fd, fcntl.F_SETFL, 0)  # set blocking

        self.pipe_abort_read_r, self.pipe_abort_read_w = None, None
        self.pipe_abort_write_r, self.pipe_abort_write_w = None, None

        try:
            self._reconfigure_port(force_update=True)

            try:
                if not self._dsrdtr:
                    self._update_dtr_state()
                if not self._rtscts:
                    self._update_rts_state()
            except IOError as e:
                # ignore Invalid argument and Inappropriate ioctl
                if e.errno not in (errno.EINVAL, errno.ENOTTY):
                    raise

            self._reset_input_buffer()

            self.pipe_abort_read_r, self.pipe_abort_read_w = os.pipe()
            self.pipe_abort_write_r, self.pipe_abort_write_w = os.pipe()
            fcntl.fcntl(self.pipe_abort_read_r, fcntl.F_SETFL, os.O_NONBLOCK)
            fcntl.fcntl(self.pipe_abort_write_r, fcntl.F_SETFL, os.O_NONBLOCK)
        except BaseException:
            try:
                os.close(self.fd)
            except Exception:
                # ignore any exception when closing the port
                # also to keep original exception that happened when setting up
                pass
            self.fd = None

            if self.pipe_abort_read_w is not None:
                os.close(self.pipe_abort_read_w)
                self.pipe_abort_read_w = None
            if self.pipe_abort_read_r is not None:
                os.close(self.pipe_abort_read_r)
                self.pipe_abort_read_r = None
            if self.pipe_abort_write_w is not None:
                os.close(self.pipe_abort_write_w)
                self.pipe_abort_write_w = None
            if self.pipe_abort_write_r is not None:
                os.close(self.pipe_abort_write_r)
                self.pipe_abort_write_r = None

            raise

        self.is_open = True

    def _reconfigure_port(self, force_update=False):
        """Set communication parameters on opened port."""
        if self.fd is None:
            raise SerialException("Can only operate on a valid file descriptor")

        # if exclusive lock is requested, create it before we modify anything else
        if self._exclusive is not None:
            if self._exclusive:
                try:
                    fcntl.flock(self.fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
                except IOError as msg:
                    raise SerialException(msg.errno, "Could not exclusively lock port {}: {}".format(self._port, msg))
            else:
                fcntl.flock(self.fd, fcntl.LOCK_UN)

        custom_baud = None

        vmin = vtime = 0                # timeout is done via select
        if self._inter_byte_timeout is not None:
            vmin = 1
            vtime = int(self._inter_byte_timeout * 10)
        try:
            orig_attr = termios.tcgetattr(self.fd)
            iflag, oflag, cflag, lflag, ispeed, ospeed, cc = orig_attr
        except termios.error as msg:      # if a port is nonexistent but has a /dev file, it'll fail here
            raise SerialException("Could not configure port: {}".format(msg))
        # set up raw mode / no echo / binary
        cflag |= (termios.CLOCAL | termios.CREAD)
        lflag &= ~(termios.ICANON | termios.ECHO | termios.ECHOE |
                   termios.ECHOK | termios.ECHONL |
                   termios.ISIG | termios.IEXTEN)  # |termios.ECHOPRT
        for flag in ('ECHOCTL', 'ECHOKE'):  # netbsd workaround for Erk
            if hasattr(termios, flag):
                lflag &= ~getattr(termios, flag)

        oflag &= ~(termios.OPOST | termios.ONLCR | termios.OCRNL)
        iflag &= ~(termios.INLCR | termios.IGNCR | termios.ICRNL | termios.IGNBRK)
        if hasattr(termios, 'IUCLC'):
            iflag &= ~termios.IUCLC
        if hasattr(termios, 'PARMRK'):
            iflag &= ~termios.PARMRK

        # setup baud rate
        try:
            ispeed = ospeed = getattr(termios, 'B{}'.format(self._baudrate))
        except AttributeError:
            try:
                ispeed = ospeed = self.BAUDRATE_CONSTANTS[self._baudrate]
            except KeyError:
                #~ raise ValueError('Invalid baud rate: %r' % self._baudrate)

                # See if BOTHER is defined for this platform; if it is, use
                # this for a speed not defined in the baudrate constants list.
                try:
                    ispeed = ospeed = BOTHER
                except NameError:
                    # may need custom baud rate, it isn't in our list.
                    ispeed = ospeed = getattr(termios, 'B38400')

                try:
                    custom_baud = int(self._baudrate)  # store for later
                except ValueError:
                    raise ValueError('Invalid baud rate: {!r}'.format(self._baudrate))
                else:
                    if custom_baud < 0:
                        raise ValueError('Invalid baud rate: {!r}'.format(self._baudrate))

        # setup char len
        cflag &= ~termios.CSIZE
        if self._bytesize == 8:
            cflag |= termios.CS8
        elif self._bytesize == 7:
            cflag |= termios.CS7
        elif self._bytesize == 6:
            cflag |= termios.CS6
        elif self._bytesize == 5:
            cflag |= termios.CS5
        else:
            raise ValueError('Invalid char len: {!r}'.format(self._bytesize))
        # setup stop bits
        if self._stopbits == serial.STOPBITS_ONE:
            cflag &= ~(termios.CSTOPB)
        elif self._stopbits == serial.STOPBITS_ONE_POINT_FIVE:
            cflag |= (termios.CSTOPB)  # XXX same as TWO.. there is no POSIX support for 1.5
        elif self._stopbits == serial.STOPBITS_TWO:
            cflag |= (termios.CSTOPB)
        else:
            raise ValueError('Invalid stop bit specification: {!r}'.format(self._stopbits))
        # setup parity
        iflag &= ~(termios.INPCK | termios.ISTRIP)
        if self._parity == serial.PARITY_NONE:
            cflag &= ~(termios.PARENB | termios.PARODD | CMSPAR)
        elif self._parity == serial.PARITY_EVEN:
            cflag &= ~(termios.PARODD | CMSPAR)
            cflag |= (termios.PARENB)
        elif self._parity == serial.PARITY_ODD:
            cflag &= ~CMSPAR
            cflag |= (termios.PARENB | termios.PARODD)
        elif self._parity == serial.PARITY_MARK and CMSPAR:
            cflag |= (termios.PARENB | CMSPAR | termios.PARODD)
        elif self._parity == serial.PARITY_SPACE and CMSPAR:
            cflag |= (termios.PARENB | CMSPAR)
            cflag &= ~(termios.PARODD)
        else:
            raise ValueError('Invalid parity: {!r}'.format(self._parity))
        # setup flow control
        # xonxoff
        if hasattr(termios, 'IXANY'):
            if self._xonxoff:
                iflag |= (termios.IXON | termios.IXOFF)  # |termios.IXANY)
            else:
                iflag &= ~(termios.IXON | termios.IXOFF | termios.IXANY)
        else:
            if self._xonxoff:
                iflag |= (termios.IXON | termios.IXOFF)
            else:
                iflag &= ~(termios.IXON | termios.IXOFF)
        # rtscts
        if hasattr(termios, 'CRTSCTS'):
            if self._rtscts:
                cflag |= (termios.CRTSCTS)
            else:
                cflag &= ~(termios.CRTSCTS)
        elif hasattr(termios, 'CNEW_RTSCTS'):   # try it with alternate constant name
            if self._rtscts:
                cflag |= (termios.CNEW_RTSCTS)
            else:
                cflag &= ~(termios.CNEW_RTSCTS)
        # XXX should there be a warning if setting up rtscts (and xonxoff etc) fails??

        # buffer
        # vmin "minimal number of characters to be read. 0 for non blocking"
        if vmin < 0 or vmin > 255:
            raise ValueError('Invalid vmin: {!r}'.format(vmin))
        cc[termios.VMIN] = vmin
        # vtime
        if vtime < 0 or vtime > 255:
            raise ValueError('Invalid vtime: {!r}'.format(vtime))
        cc[termios.VTIME] = vtime
        # activate settings
        if force_update or [iflag, oflag, cflag, lflag, ispeed, ospeed, cc] != orig_attr:
            termios.tcsetattr(
                self.fd,
                termios.TCSANOW,
                [iflag, oflag, cflag, lflag, ispeed, ospeed, cc])

        # apply custom baud rate, if any
        if custom_baud is not None:
            self._set_special_baudrate(custom_baud)

        if self._rs485_mode is not None:
            self._set_rs485_mode(self._rs485_mode)

    def close(self):
        """Close port"""
        if self.is_open:
            if self.fd is not None:
                os.close(self.fd)
                self.fd = None
                os.close(self.pipe_abort_read_w)
                os.close(self.pipe_abort_read_r)
                os.close(self.pipe_abort_write_w)
                os.close(self.pipe_abort_write_r)
                self.pipe_abort_read_r, self.pipe_abort_read_w = None, None
                self.pipe_abort_write_r, self.pipe_abort_write_w = None, None
            self.is_open = False

    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -

    @property
    def in_waiting(self):
        """Return the number of bytes currently in the input buffer."""
        #~ s = fcntl.ioctl(self.fd, termios.FIONREAD, TIOCM_zero_str)
        s = fcntl.ioctl(self.fd, TIOCINQ, TIOCM_zero_str)
        return struct.unpack('I', s)[0]

    # select based implementation, proved to work on many systems
    def read(self, size=1):
        """\
        Read size bytes from the serial port. If a timeout is set it may
        return less characters as requested. With no timeout it will block
        until the requested number of bytes is read.
        """
        if not self.is_open:
            raise PortNotOpenError()
        read = bytearray()
        timeout = Timeout(self._timeout)
        while len(read) < size:
            try:
                ready, _, _ = select.select([self.fd, self.pipe_abort_read_r], [], [], timeout.time_left())
                if self.pipe_abort_read_r in ready:
                    os.read(self.pipe_abort_read_r, 1000)
                    break
                # If select was used with a timeout, and the timeout occurs, it
                # returns with empty lists -> thus abort read operation.
                # For timeout == 0 (non-blocking operation) also abort when
                # there is nothing to read.
                if not ready:
                    break   # timeout
                buf = os.read(self.fd, size - len(read))
            except OSError as e:
                # this is for Python 3.x where select.error is a subclass of
                # OSError ignore BlockingIOErrors and EINTR. other errors are shown
                # https://www.python.org/dev/peps/pep-0475.
                if e.errno not in (errno.EAGAIN, errno.EALREADY, errno.EWOULDBLOCK, errno.EINPROGRESS, errno.EINTR):
                    raise SerialException('read failed: {}'.format(e))
            except select.error as e:
                # this is for Python 2.x
                # ignore BlockingIOErrors and EINTR. all errors are shown
                # see also http://www.python.org/dev/peps/pep-3151/#select
                if e[0] not in (errno.EAGAIN, errno.EALREADY, errno.EWOULDBLOCK, errno.EINPROGRESS, errno.EINTR):
                    raise SerialException('read failed: {}'.format(e))
            else:
                # read should always return some data as select reported it was
                # ready to read when we get to this point.
                if not buf:
                    # Disconnected devices, at least on Linux, show the
                    # behavior that they are always ready to read immediately
                    # but reading returns nothing.
                    raise SerialException(
                        'device reports readiness to read but returned no data '
                        '(device disconnected or multiple access on port?)')
                read.extend(buf)

            if timeout.expired():
                break
        return bytes(read)

    def cancel_read(self):
        if self.is_open:
            os.write(self.pipe_abort_read_w, b"x")

    def cancel_write(self):
        if self.is_open:
            os.write(self.pipe_abort_write_w, b"x")

    def write(self, data):
        """Output the given byte string over the serial port."""
        if not self.is_open:
            raise PortNotOpenError()
        d = to_bytes(data)
        tx_len = length = len(d)
        timeout = Timeout(self._write_timeout)
        while tx_len > 0:
            try:
                n = os.write(self.fd, d)
                if timeout.is_non_blocking:
                    # Zero timeout indicates non-blocking - simply return the
                    # number of bytes of data actually written
                    return n
                elif not timeout.is_infinite:
                    # when timeout is set, use select to wait for being ready
                    # with the time left as timeout
                    if timeout.expired():
                        raise SerialTimeoutException('Write timeout')
                    abort, ready, _ = select.select([self.pipe_abort_write_r], [self.fd], [], timeout.time_left())
                    if abort:
                        os.read(self.pipe_abort_write_r, 1000)
                        break
                    if not ready:
                        raise SerialTimeoutException('Write timeout')
                else:
                    assert timeout.time_left() is None
                    # wait for write operation
                    abort, ready, _ = select.select([self.pipe_abort_write_r], [self.fd], [], None)
                    if abort:
                        os.read(self.pipe_abort_write_r, 1)
                        break
                    if not ready:
                        raise SerialException('write failed (select)')
                d = d[n:]
                tx_len -= n
            except SerialException:
                raise
            except OSError as e:
                # this is for Python 3.x where select.error is a subclass of
                # OSError ignore BlockingIOErrors and EINTR. other errors are shown
                # https://www.python.org/dev/peps/pep-0475.
                if e.errno not in (errno.EAGAIN, errno.EALREADY, errno.EWOULDBLOCK, errno.EINPROGRESS, errno.EINTR):
                    raise SerialException('write failed: {}'.format(e))
            except select.error as e:
                # this is for Python 2.x
                # ignore BlockingIOErrors and EINTR. all errors are shown
                # see also http://www.python.org/dev/peps/pep-3151/#select
                if e[0] not in (errno.EAGAIN, errno.EALREADY, errno.EWOULDBLOCK, errno.EINPROGRESS, errno.EINTR):
                    raise SerialException('write failed: {}'.format(e))
            if not timeout.is_non_blocking and timeout.expired():
                raise SerialTimeoutException('Write timeout')
        return length - len(d)

    def flush(self):
        """\
        Flush of file like objects. In this case, wait until all data
        is written.
        """
        if not self.is_open:
            raise PortNotOpenError()
        termios.tcdrain(self.fd)

    def _reset_input_buffer(self):
        """Clear input buffer, discarding all that is in the buffer."""
        termios.tcflush(self.fd, termios.TCIFLUSH)

    def reset_input_buffer(self):
        """Clear input buffer, discarding all that is in the buffer."""
        if not self.is_open:
            raise PortNotOpenError()
        self._reset_input_buffer()

    def reset_output_buffer(self):
        """\
        Clear output buffer, aborting the current output and discarding all
        that is in the buffer.
        """
        if not self.is_open:
            raise PortNotOpenError()
        termios.tcflush(self.fd, termios.TCOFLUSH)

    def send_break(self, duration=0.25):
        """\
        Send break condition. Timed, returns to idle state after given
        duration.
        """
        if not self.is_open:
            raise PortNotOpenError()
        termios.tcsendbreak(self.fd, int(duration / 0.25))

    def _update_rts_state(self):
        """Set terminal status line: Request To Send"""
        if self._rts_state:
            fcntl.ioctl(self.fd, TIOCMBIS, TIOCM_RTS_str)
        else:
            fcntl.ioctl(self.fd, TIOCMBIC, TIOCM_RTS_str)

    def _update_dtr_state(self):
        """Set terminal status line: Data Terminal Ready"""
        if self._dtr_state:
            fcntl.ioctl(self.fd, TIOCMBIS, TIOCM_DTR_str)
        else:
            fcntl.ioctl(self.fd, TIOCMBIC, TIOCM_DTR_str)

    @property
    def cts(self):
        """Read terminal status line: Clear To Send"""
        if not self.is_open:
            raise PortNotOpenError()
        s = fcntl.ioctl(self.fd, TIOCMGET, TIOCM_zero_str)
        return struct.unpack('I', s)[0] & TIOCM_CTS != 0

    @property
    def dsr(self):
        """Read terminal status line: Data Set Ready"""
        if not self.is_open:
            raise PortNotOpenError()
        s = fcntl.ioctl(self.fd, TIOCMGET, TIOCM_zero_str)
        return struct.unpack('I', s)[0] & TIOCM_DSR != 0

    @property
    def ri(self):
        """Read terminal status line: Ring Indicator"""
        if not self.is_open:
            raise PortNotOpenError()
        s = fcntl.ioctl(self.fd, TIOCMGET, TIOCM_zero_str)
        return struct.unpack('I', s)[0] & TIOCM_RI != 0

    @property
    def cd(self):
        """Read terminal status line: Carrier Detect"""
        if not self.is_open:
            raise PortNotOpenError()
        s = fcntl.ioctl(self.fd, TIOCMGET, TIOCM_zero_str)
        return struct.unpack('I', s)[0] & TIOCM_CD != 0

    # - - platform specific - - - -

    @property
    def out_waiting(self):
        """Return the number of bytes currently in the output buffer."""
        #~ s = fcntl.ioctl(self.fd, termios.FIONREAD, TIOCM_zero_str)
        s = fcntl.ioctl(self.fd, TIOCOUTQ, TIOCM_zero_str)
        return struct.unpack('I', s)[0]

    def fileno(self):
        """\
        For easier use of the serial port instance with select.
        WARNING: this function is not portable to different platforms!
        """
        if not self.is_open:
            raise PortNotOpenError()
        return self.fd

    def set_input_flow_control(self, enable=True):
        """\
        Manually control flow - when software flow control is enabled.
        This will send XON (true) or XOFF (false) to the other device.
        WARNING: this function is not portable to different platforms!
        """
        if not self.is_open:
            raise PortNotOpenError()
        if enable:
            termios.tcflow(self.fd, termios.TCION)
        else:
            termios.tcflow(self.fd, termios.TCIOFF)

    def set_output_flow_control(self, enable=True):
        """\
        Manually control flow of outgoing data - when hardware or software flow
        control is enabled.
        WARNING: this function is not portable to different platforms!
        """
        if not self.is_open:
            raise PortNotOpenError()
        if enable:
            termios.tcflow(self.fd, termios.TCOON)
        else:
            termios.tcflow(self.fd, termios.TCOOFF)

    def nonblocking(self):
        """DEPRECATED - has no use"""
        import warnings
        warnings.warn("nonblocking() has no effect, already nonblocking", DeprecationWarning)


class PosixPollSerial(Serial):
    """\
    Poll based read implementation. Not all systems support poll properly.
    However this one has better handling of errors, such as a device
    disconnecting while it's in use (e.g. USB-serial unplugged).
    """

    def read(self, size=1):
        """\
        Read size bytes from the serial port. If a timeout is set it may
        return less characters as requested. With no timeout it will block
        until the requested number of bytes is read.
        """
        if not self.is_open:
            raise PortNotOpenError()
        read = bytearray()
        timeout = Timeout(self._timeout)
        poll = select.poll()
        poll.register(self.fd, select.POLLIN | select.POLLERR | select.POLLHUP | select.POLLNVAL)
        poll.register(self.pipe_abort_read_r, select.POLLIN | select.POLLERR | select.POLLHUP | select.POLLNVAL)
        if size > 0:
            while len(read) < size:
                # print "\tread(): size",size, "have", len(read)    #debug
                # wait until device becomes ready to read (or something fails)
                for fd, event in poll.poll(None if timeout.is_infinite else (timeout.time_left() * 1000)):
                    if fd == self.pipe_abort_read_r:
                        break
                    if event & (select.POLLERR | select.POLLHUP | select.POLLNVAL):
                        raise SerialException('device reports error (poll)')
                    #  we don't care if it is select.POLLIN or timeout, that's
                    #  handled below
                if fd == self.pipe_abort_read_r:
                    os.read(self.pipe_abort_read_r, 1000)
                    break
                buf = os.read(self.fd, size - len(read))
                read.extend(buf)
                if timeout.expired() \
                        or (self._inter_byte_timeout is not None and self._inter_byte_timeout > 0) and not buf:
                    break   # early abort on timeout
        return bytes(read)


class VTIMESerial(Serial):
    """\
    Implement timeout using vtime of tty device instead of using select.
    This means that no inter character timeout can be specified and that
    the error handling is degraded.

    Overall timeout is disabled when inter-character timeout is used.

    Note that this implementation does NOT support cancel_read(), it will
    just ignore that.
    """

    def _reconfigure_port(self, force_update=True):
        """Set communication parameters on opened port."""
        super(VTIMESerial, self)._reconfigure_port()
        fcntl.fcntl(self.fd, fcntl.F_SETFL, 0)  # clear O_NONBLOCK

        if self._inter_byte_timeout is not None:
            vmin = 1
            vtime = int(self._inter_byte_timeout * 10)
        elif self._timeout is None:
            vmin = 1
            vtime = 0
        else:
            vmin = 0
            vtime = int(self._timeout * 10)
        try:
            orig_attr = termios.tcgetattr(self.fd)
            iflag, oflag, cflag, lflag, ispeed, ospeed, cc = orig_attr
        except termios.error as msg:      # if a port is nonexistent but has a /dev file, it'll fail here
            raise serial.SerialException("Could not configure port: {}".format(msg))

        if vtime < 0 or vtime > 255:
            raise ValueError('Invalid vtime: {!r}'.format(vtime))
        cc[termios.VTIME] = vtime
        cc[termios.VMIN] = vmin

        termios.tcsetattr(
                self.fd,
                termios.TCSANOW,
                [iflag, oflag, cflag, lflag, ispeed, ospeed, cc])

    def read(self, size=1):
        """\
        Read size bytes from the serial port. If a timeout is set it may
        return less characters as requested. With no timeout it will block
        until the requested number of bytes is read.
        """
        if not self.is_open:
            raise PortNotOpenError()
        read = bytearray()
        while len(read) < size:
            buf = os.read(self.fd, size - len(read))
            if not buf:
                break
            read.extend(buf)
        return bytes(read)

    # hack to make hasattr return false
    cancel_read = property()



import copy
from typing import (
    List,
    Optional,
)

import cfgtools
from mw_helper import get_modifier_prefixes

from util import find_key_value

from ._BumpBar import BumpBar
from ._KdsView import KdsView
from ._Modifier import Modifier


def get_first_of(it, default_value=None):
    return next(it, default_value)


class Kds(object):
    def __init__(
        self,
        kds_config,  # type: cfgtools.Group
        kds_views,   # type: List[KdsView]
        brands       # type: Optional[List]
    ):
        # type: (...) -> None

        self.name = kds_config.name
        self.title = ''
        self.language = ''
        self.bump_bar = None
        self.modifiers = []
        self.views = []
        self.brands = brands

        self.parse_kds(kds_config, kds_views)

    def set_brands(self, brands):
        self.brands = brands

    def parse_kds(self, kds_config, kds_views):
        self.language = find_key_value(kds_config, "Language")
        self.title = find_key_value(kds_config, "Title")
        self.modifiers = self._get_global_modifiers()

        for group in kds_config.groups:
            if group.name == 'BumpBar':
                self.bump_bar = BumpBar(group)
            elif group.name == 'ModifierPrefixes':
                self.modifiers.extend(self.get_modifiers(group=group))
            elif group.name == 'Views':
                for view_group in group.groups:
                    view_name = get_first_of(x for x in view_group.keys if x.name == "ViewName").values[0]
                    view = copy.deepcopy(get_first_of(x for x in kds_views if x.name == view_name))
                    view.name = view_group.name
                    view.display_mode = get_first_of(x for x in view_group.keys if x.name == "Display").values[0]
                    view.title = get_first_of(x for x in view_group.keys if x.name == "Title").values[0]
                    self.views.append(view)

                self.views[0].selected = True

    @staticmethod
    def get_modifiers(group):
        modifiers = []
        for modifier_key in group.keys:
            modifiers.append(Modifier(modifier_key))

        return modifiers

    @staticmethod
    def _get_global_modifiers():
        # type: (...) -> List

        modifiers = []
        global_modifiers = get_modifier_prefixes()

        for global_modifier in global_modifiers.items():
            modifiers.append((Modifier(
                key=global_modifier,
                from_tuple=True,
            )))

        return modifiers



import json
import logging

from datetime import datetime

from msgbus import (
    FM_PARAM,
    TK_POS_BUSINESSEND,
    TK_SYS_ACK,
)
from helper import translate_pos_label
from mw_helper import show_message_dialog, show_message_options_dialog
from sysactions import (
    action,
    get_business_period,
    get_model,
    get_poslist,
    print_report,
    send_message,
    set_custom,
    translate_message,
)
from systools import sys_log_info, sys_log_error, sys_log_exception
from bustoken import (
    TK_POS_HAS_BINARY_UPDATE_TO_APPLY,
    TK_POS_UPDATER_PERFORM_BINARIES_UPDATE,
)
from actions.util import list_opened_users, get_operator_name
from actions.pos.check_table_status import has_tab_or_tables_opened, show_open_table_dialog
from actions.util.get_menu_user_logged import get_menu_manager_authenticated_user
from actions.models.mw_sys_errors import MwSysErrors
from application.interactor import MaintenanceInteractor

from .. import mb_context

logger = logging.getLogger("AcceptUpdateBinary")
pos_list = ()


@action
def close_day(
    pos_id,                         # type: str
    pos_id_to_show_message=None,    # type: str
):
    # type: (...) -> bool

    if pos_id_to_show_message is None:
        pos_id_to_show_message = pos_id
    from msgbusboundary import MsgBusTableService as tableService

    error_dict = {}
    model = get_model(pos_id)
    period = get_business_period(model)
    is_table_service = model.find("WorkingMode").get("usrCtrlType") == "TS"

    close_day_pos_list = [pos_id]
    if is_table_service:
        close_day_pos_list = tableService().get_exclusive_pos_list(user_control_type="TS")

    if is_table_service and has_tab_or_tables_opened(period, pos_id):
        show_open_table_dialog(pos_id=pos_id_to_show_message)
        return False

    opened_users = list_opened_users(pos_id)
    _fill_pos_list_with_can_close_day_pos(error_dict, period, close_day_pos_list, opened_users)
    error_messages = _parse_error_messages(model, error_dict)

    if len(error_messages) > 0:
        show_message_dialog(
            pos_id=pos_id_to_show_message,
            message="$CANNOT_CLOSE_DAYS|{}".format("\\".join(error_messages)),
        )
        return False

    if len(close_day_pos_list) == 0:
        show_message_dialog(
            pos_id=pos_id_to_show_message,
            message="$HAS_NO_POS_TO_CLOSE",
        )
        return False

    close_date = datetime.strptime(period, '%Y%m%d').strftime('%d/%m/%Y')
    close_day_pos_list = [int(x) for x in close_day_pos_list]
    pos = translate_pos_label(model=model)
    confirmation_message = "$CONFIRM_CLOSE_DAY|{0}|{1}".format(close_date, "{} {}".format(pos, close_day_pos_list))
    resp = show_message_options_dialog(
        pos_id=pos_id_to_show_message,
        buttons="$OK|$PREVIOUS",
        title="$INFORMATION",
        message=confirmation_message,
    )

    if resp is None or resp == 1:
        return False

    try:
        sys_log_info("Closing business day for POS's: {}".format(close_day_pos_list))
        error_dict = {}
        succeed_pos_id = []

        for pos_no in close_day_pos_list:
            _perform_close_day(pos_no, succeed_pos_id, error_dict)
            set_custom(pos_no, 'POS_MUST_BE_BLOCKED', False)

        if len(error_dict) > 0:
            error_messages = _parse_error_messages(model, error_dict)
            show_message_dialog(
                pos_id=pos_id_to_show_message,
                title="$INFORMATION",
                message="$ERROR_CLOSING_DAYS|{}".format("\\".join(error_messages)),
            )

        if len(succeed_pos_id) > 0:
            report_pos_id = "|".join(map(str, succeed_pos_id))
            menu_manager_user = get_menu_manager_authenticated_user(int(pos_id))
            print_report(
                pos_id_to_show_message,
                model,
                True,
                "cash",
                pos_id,
                period,
                "0",
                "true",
                report_pos_id,
                "end_of_day",
                "",
                menu_manager_user,
            )

            if _is_last_pos(period):
                maintenance_interactor = MaintenanceInteractor()
                maintenance_interactor.verify_clean_databases(period, pos_id)

                _verify_store_close_event(period)

            _verify_updates(menu_manager_user, period, pos_id)

    except Exception as ex:
        show_message_dialog(
            pos_id=pos_id_to_show_message,
            title="$ERROR",
            message="$ERROR_CLOSING_DAY|{}".format(ex.message),
        )


def _is_last_pos(period):
    global pos_list
    if pos_list is None or len(pos_list) == 0:
        pos_list = get_poslist()

    for pos_no in pos_list:
        model = get_model(pos_no)
        if model.find("PosState").get("period") == period and model.find("PosState").get("state") != "CLOSED":
            return False

    return True


def _verify_store_close_event(period):
    global pos_list
    if pos_list is None or len(pos_list) == 0:
        pos_list = get_poslist()

    for pos_no in pos_list:
        model = get_model(pos_no)
        if model.find("PosState").get("period") == period and model.find("PosState").get("state") != "CLOSED":
            break
    else:
        event_xml = '''<STORE_CLOSE businessPeriod="{}"/>'''
        data = event_xml.format(period)

        mb_context.MB_EasyEvtSend(
            subject="STORE_STATUS",
            type="CLOSE_STORE_DAY",
            xml=data,
            synchronous=False,
            timeout=-1
        )


def _verify_updates(menu_manager_user, period, pos_id):
    pending_updates = _has_pending_pos_update(pos_id)
    update_response = json.loads(pending_updates)
    update_type = str(update_response.get("update"))

    if update_type not in ["HAS_PENDING_UPDATE", "UPDATE_NOW"]:
        return

    if check_has_opened_day_others_pos(int(pos_id), period):
        show_message_dialog(pos_id, message="$HAS_OTHERS_POS_OPENED")
        return

    if update_type == "HAS_PENDING_UPDATE":
        confirmation_message = "$HAS_PENDING_UPDATES"
        resp = show_message_options_dialog(
            pos_id=pos_id,
            buttons="$OK|$PREVIOUS",
            title="$INFORMATION",
            message=confirmation_message,
        )
        if resp is not None and resp == 0:
            manager_info = "{} - {}".format(str(menu_manager_user), get_operator_name(menu_manager_user))
            last_version = update_response.get("lastVersion")
            current_version = update_response.get("currentVersion")
            _audit_logger_notify(pos_id, manager_info, last_version, current_version)
            _apply_binaries_update(pos_id)

    elif update_type == "UPDATE_NOW":
        confirmation_message = "$HAS_PENDING_UPDATES_TO_APPLY_NOW"
        show_message_dialog(pos_id, "$INFORMATION", confirmation_message)
        _apply_binaries_update(pos_id)


def _has_pending_pos_update(pos_id):
    msg = send_message("PosUpdater", TK_POS_HAS_BINARY_UPDATE_TO_APPLY, timeout=60 * 1000000)
    if msg.token != TK_SYS_ACK:
        sys_log_error("Error getting update binaries status on pos id: {}. Error: {}".format(pos_id, msg.data))

    return msg.data


def _apply_binaries_update(pos_id):
    msg = send_message(
        "PosUpdater",
        TK_POS_UPDATER_PERFORM_BINARIES_UPDATE,
        FM_PARAM,
        "updateNow",
        timeout=60 * 1000000
    )
    if msg.token == TK_SYS_ACK:
        return msg.data
    else:
        sys_log_error("Error applying update binaries status on pos id: {}. Error: {}".format(pos_id, msg.data))


def _fill_pos_list_with_can_close_day_pos(error_dict, period, pos_list, opened_users):
    for pos_no in sorted(pos_list):
        model = get_model(pos_no)
        if not _verify_if_can_close_day(pos_no, model, period, error_dict, opened_users):
            pos_list.remove(pos_no)


def _parse_error_messages(model, error_dict):
    error_messages = []
    if len(error_dict) > 0:
        for key in error_dict:
            if key in MwSysErrors.list_values():
                msg = translate_message(model, MwSysErrors.get_name(key))
            else:
                msg = str(key)

            pos = translate_pos_label(model=model)
            error_messages.append("{} {}".format(pos, error_dict[key]) + (" - {}".format(msg)))
    return error_messages


def _perform_close_day(pos_id, succeed_pos, error_dict):
    try:
        sys_log_info("Closing business day on POS id: [{}]".format(pos_id))

        pos_model = get_model(pos_id)
        if pos_model.find("PosState").get("state") == "CLOSED":
            return

        msg = send_message("POS%d" % int(pos_id), TK_POS_BUSINESSEND, FM_PARAM, str(pos_id), timeout=60 * 1000000)
        if msg.token == TK_SYS_ACK:
            succeed_pos.append(pos_id)
            return
        else:
            sys_log_error("Error closing business day on pos id: {}. Error: {}".format(pos_id, msg.data))
            _fill_error_dict(error_dict, pos_id, msg.data)

    except Exception as ex:
        sys_log_exception("Error closing business day on pos id: {}".format(pos_id))
        _fill_error_dict(error_dict, pos_id, ex.message)


def _verify_if_can_close_day(pos_id, model, period, error_dict, opened_users):
    try:
        if model.find("PosState").get("state") == "CLOSED":
            error_msg = translate_message(model, "DAY_IS_ALREADY_CLOSED")
            _fill_error_dict(error_dict, pos_id, error_msg)
            return False

        operator = model.find("Operator")
        if operator is not None and operator.get('id') in [x.get('id') for x in opened_users]:
            error_msg = translate_message(model, "THERE_ARE_OPEN_OPERATORS")
            _fill_error_dict(error_dict, pos_id, error_msg)
            return False

        if period == '0':
            error_msg = translate_message(model, "NEED_TO_OPEN_DAY_FIRST")
            _fill_error_dict(error_dict, pos_id, error_msg)
            return False
    except Exception as ex:
        _fill_error_dict(error_dict, pos_id, ex.message)

    return True


def _fill_error_dict(error_dict, pos_id, error_message):
    error_message = error_message.split("\0")[0]

    if error_message not in error_dict:
        error_dict[error_message] = [pos_id]
    else:
        error_dict[error_message].append(pos_id)

    return error_dict


def _audit_logger_notify(
    pos_id,                 # type: int
    user_id,                # type: str
    old_version=None,       # type: str
    current_version=None    # type: str
):
    # type: (...)-> None

    event_xml = '''<UPDATE posId="{}" userId="{}" oldVersion="{}" newVersion="{}"/>'''
    data = event_xml.format(pos_id, user_id, old_version, current_version)
    logger.info("User Accept update binary: {}".format(data))

    mb_context.MB_EasyEvtSend(
        subject="POS_UPDATER_LOGGER",
        type="UPDATE_ACCEPTED",
        xml=data,
        synchronous=False,
        timeout=-1
    )


def check_has_opened_day_others_pos(
    current_pos_id,         # type: int
    close_business_period   # type: str
):
    # type: (...) -> bool

    for pos_id in get_poslist():
        if pos_id in [0, current_pos_id]:
            continue

        model = get_model(pos_id)
        working_mode = model.find("WorkingMode")
        pod_type = working_mode.get("podType") if working_mode is not None else None
        if pod_type == "TT":
            continue

        model_pos_state = model.find("PosState")
        pos_state = model_pos_state.get("state") if model_pos_state is not None else None
        if pos_state != "OPENED":
            continue

        pos_business_period = get_business_period(model)
        if pos_business_period >= close_business_period and pos_state:
            return True

    return False



class AuthorizationResponse(object):
    def __init__(
            self,
            success,                # type: bool
            authorized_by,          # type: str
            authorized_by_level     # type: str
    ):
        # type: (...) -> None

        self.success = success
        self.authorized_by = authorized_by
        self.authorized_by_level = authorized_by_level





from _UpdateTotalSoldInteractor import UpdateTotalSoldInteractor
from _UpdateTotalSoldOrderInteractor import UpdateTotalSoldOrderInteractor
from _UpdateTotalSoldTableInteractor import UpdateTotalSoldTableInteractor



# coding: utf-8

"""
ASN.1 type classes for the online certificate status protocol (OCSP). Exports
the following items:

 - OCSPRequest()
 - OCSPResponse()

Other type classes are defined that help compose the types listed above.
"""

from __future__ import unicode_literals, division, absolute_import, print_function

from ._errors import unwrap
from .algos import DigestAlgorithm, SignedDigestAlgorithm
from .core import (
    Boolean,
    Choice,
    Enumerated,
    GeneralizedTime,
    IA5String,
    Integer,
    Null,
    ObjectIdentifier,
    OctetBitString,
    OctetString,
    ParsableOctetString,
    Sequence,
    SequenceOf,
)
from .crl import AuthorityInfoAccessSyntax, CRLReason
from .keys import PublicKeyAlgorithm
from .x509 import Certificate, GeneralName, GeneralNames, Name


# The structures in this file are taken from https://tools.ietf.org/html/rfc6960


class Version(Integer):
    _map = {
        0: 'v1'
    }


class CertId(Sequence):
    _fields = [
        ('hash_algorithm', DigestAlgorithm),
        ('issuer_name_hash', OctetString),
        ('issuer_key_hash', OctetString),
        ('serial_number', Integer),
    ]


class ServiceLocator(Sequence):
    _fields = [
        ('issuer', Name),
        ('locator', AuthorityInfoAccessSyntax),
    ]


class RequestExtensionId(ObjectIdentifier):
    _map = {
        '1.3.6.1.5.5.7.48.1.7': 'service_locator',
    }


class RequestExtension(Sequence):
    _fields = [
        ('extn_id', RequestExtensionId),
        ('critical', Boolean, {'default': False}),
        ('extn_value', ParsableOctetString),
    ]

    _oid_pair = ('extn_id', 'extn_value')
    _oid_specs = {
        'service_locator': ServiceLocator,
    }


class RequestExtensions(SequenceOf):
    _child_spec = RequestExtension


class Request(Sequence):
    _fields = [
        ('req_cert', CertId),
        ('single_request_extensions', RequestExtensions, {'explicit': 0, 'optional': True}),
    ]

    _processed_extensions = False
    _critical_extensions = None
    _service_locator_value = None

    def _set_extensions(self):
        """
        Sets common named extensions to private attributes and creates a list
        of critical extensions
        """

        self._critical_extensions = set()

        for extension in self['single_request_extensions']:
            name = extension['extn_id'].native
            attribute_name = '_%s_value' % name
            if hasattr(self, attribute_name):
                setattr(self, attribute_name, extension['extn_value'].parsed)
            if extension['critical'].native:
                self._critical_extensions.add(name)

        self._processed_extensions = True

    @property
    def critical_extensions(self):
        """
        Returns a set of the names (or OID if not a known extension) of the
        extensions marked as critical

        :return:
            A set of unicode strings
        """

        if not self._processed_extensions:
            self._set_extensions()
        return self._critical_extensions

    @property
    def service_locator_value(self):
        """
        This extension is used when communicating with an OCSP responder that
        acts as a proxy for OCSP requests

        :return:
            None or a ServiceLocator object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._service_locator_value


class Requests(SequenceOf):
    _child_spec = Request


class ResponseType(ObjectIdentifier):
    _map = {
        '1.3.6.1.5.5.7.48.1.1': 'basic_ocsp_response',
    }


class AcceptableResponses(SequenceOf):
    _child_spec = ResponseType


class PreferredSignatureAlgorithm(Sequence):
    _fields = [
        ('sig_identifier', SignedDigestAlgorithm),
        ('cert_identifier', PublicKeyAlgorithm, {'optional': True}),
    ]


class PreferredSignatureAlgorithms(SequenceOf):
    _child_spec = PreferredSignatureAlgorithm


class TBSRequestExtensionId(ObjectIdentifier):
    _map = {
        '1.3.6.1.5.5.7.48.1.2': 'nonce',
        '1.3.6.1.5.5.7.48.1.4': 'acceptable_responses',
        '1.3.6.1.5.5.7.48.1.8': 'preferred_signature_algorithms',
    }


class TBSRequestExtension(Sequence):
    _fields = [
        ('extn_id', TBSRequestExtensionId),
        ('critical', Boolean, {'default': False}),
        ('extn_value', ParsableOctetString),
    ]

    _oid_pair = ('extn_id', 'extn_value')
    _oid_specs = {
        'nonce': OctetString,
        'acceptable_responses': AcceptableResponses,
        'preferred_signature_algorithms': PreferredSignatureAlgorithms,
    }


class TBSRequestExtensions(SequenceOf):
    _child_spec = TBSRequestExtension


class TBSRequest(Sequence):
    _fields = [
        ('version', Version, {'explicit': 0, 'default': 'v1'}),
        ('requestor_name', GeneralName, {'explicit': 1, 'optional': True}),
        ('request_list', Requests),
        ('request_extensions', TBSRequestExtensions, {'explicit': 2, 'optional': True}),
    ]


class Certificates(SequenceOf):
    _child_spec = Certificate


class Signature(Sequence):
    _fields = [
        ('signature_algorithm', SignedDigestAlgorithm),
        ('signature', OctetBitString),
        ('certs', Certificates, {'explicit': 0, 'optional': True}),
    ]


class OCSPRequest(Sequence):
    _fields = [
        ('tbs_request', TBSRequest),
        ('optional_signature', Signature, {'explicit': 0, 'optional': True}),
    ]

    _processed_extensions = False
    _critical_extensions = None
    _nonce_value = None
    _acceptable_responses_value = None
    _preferred_signature_algorithms_value = None

    def _set_extensions(self):
        """
        Sets common named extensions to private attributes and creates a list
        of critical extensions
        """

        self._critical_extensions = set()

        for extension in self['tbs_request']['request_extensions']:
            name = extension['extn_id'].native
            attribute_name = '_%s_value' % name
            if hasattr(self, attribute_name):
                setattr(self, attribute_name, extension['extn_value'].parsed)
            if extension['critical'].native:
                self._critical_extensions.add(name)

        self._processed_extensions = True

    @property
    def critical_extensions(self):
        """
        Returns a set of the names (or OID if not a known extension) of the
        extensions marked as critical

        :return:
            A set of unicode strings
        """

        if not self._processed_extensions:
            self._set_extensions()
        return self._critical_extensions

    @property
    def nonce_value(self):
        """
        This extension is used to prevent replay attacks by including a unique,
        random value with each request/response pair

        :return:
            None or an OctetString object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._nonce_value

    @property
    def acceptable_responses_value(self):
        """
        This extension is used to allow the client and server to communicate
        with alternative response formats other than just basic_ocsp_response,
        although no other formats are defined in the standard.

        :return:
            None or an AcceptableResponses object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._acceptable_responses_value

    @property
    def preferred_signature_algorithms_value(self):
        """
        This extension is used by the client to define what signature algorithms
        are preferred, including both the hash algorithm and the public key
        algorithm, with a level of detail down to even the public key algorithm
        parameters, such as curve name.

        :return:
            None or a PreferredSignatureAlgorithms object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._preferred_signature_algorithms_value


class OCSPResponseStatus(Enumerated):
    _map = {
        0: 'successful',
        1: 'malformed_request',
        2: 'internal_error',
        3: 'try_later',
        5: 'sign_required',
        6: 'unauthorized',
    }


class ResponderId(Choice):
    _alternatives = [
        ('by_name', Name, {'explicit': 1}),
        ('by_key', OctetString, {'explicit': 2}),
    ]


# Custom class to return a meaningful .native attribute from CertStatus()
class StatusGood(Null):
    def set(self, value):
        """
        Sets the value of the object

        :param value:
            None or 'good'
        """

        if value is not None and value != 'good' and not isinstance(value, Null):
            raise ValueError(unwrap(
                '''
                value must be one of None, "good", not %s
                ''',
                repr(value)
            ))

        self.contents = b''

    @property
    def native(self):
        return 'good'


# Custom class to return a meaningful .native attribute from CertStatus()
class StatusUnknown(Null):
    def set(self, value):
        """
        Sets the value of the object

        :param value:
            None or 'unknown'
        """

        if value is not None and value != 'unknown' and not isinstance(value, Null):
            raise ValueError(unwrap(
                '''
                value must be one of None, "unknown", not %s
                ''',
                repr(value)
            ))

        self.contents = b''

    @property
    def native(self):
        return 'unknown'


class RevokedInfo(Sequence):
    _fields = [
        ('revocation_time', GeneralizedTime),
        ('revocation_reason', CRLReason, {'explicit': 0, 'optional': True}),
    ]


class CertStatus(Choice):
    _alternatives = [
        ('good', StatusGood, {'implicit': 0}),
        ('revoked', RevokedInfo, {'implicit': 1}),
        ('unknown', StatusUnknown, {'implicit': 2}),
    ]


class CrlId(Sequence):
    _fields = [
        ('crl_url', IA5String, {'explicit': 0, 'optional': True}),
        ('crl_num', Integer, {'explicit': 1, 'optional': True}),
        ('crl_time', GeneralizedTime, {'explicit': 2, 'optional': True}),
    ]


class SingleResponseExtensionId(ObjectIdentifier):
    _map = {
        '1.3.6.1.5.5.7.48.1.3': 'crl',
        '1.3.6.1.5.5.7.48.1.6': 'archive_cutoff',
        # These are CRLEntryExtension values from
        # https://tools.ietf.org/html/rfc5280
        '2.5.29.21': 'crl_reason',
        '2.5.29.24': 'invalidity_date',
        '2.5.29.29': 'certificate_issuer',
        # https://tools.ietf.org/html/rfc6962.html#page-13
        '1.3.6.1.4.1.11129.2.4.5': 'signed_certificate_timestamp_list',
    }


class SingleResponseExtension(Sequence):
    _fields = [
        ('extn_id', SingleResponseExtensionId),
        ('critical', Boolean, {'default': False}),
        ('extn_value', ParsableOctetString),
    ]

    _oid_pair = ('extn_id', 'extn_value')
    _oid_specs = {
        'crl': CrlId,
        'archive_cutoff': GeneralizedTime,
        'crl_reason': CRLReason,
        'invalidity_date': GeneralizedTime,
        'certificate_issuer': GeneralNames,
        'signed_certificate_timestamp_list': OctetString,
    }


class SingleResponseExtensions(SequenceOf):
    _child_spec = SingleResponseExtension


class SingleResponse(Sequence):
    _fields = [
        ('cert_id', CertId),
        ('cert_status', CertStatus),
        ('this_update', GeneralizedTime),
        ('next_update', GeneralizedTime, {'explicit': 0, 'optional': True}),
        ('single_extensions', SingleResponseExtensions, {'explicit': 1, 'optional': True}),
    ]

    _processed_extensions = False
    _critical_extensions = None
    _crl_value = None
    _archive_cutoff_value = None
    _crl_reason_value = None
    _invalidity_date_value = None
    _certificate_issuer_value = None

    def _set_extensions(self):
        """
        Sets common named extensions to private attributes and creates a list
        of critical extensions
        """

        self._critical_extensions = set()

        for extension in self['single_extensions']:
            name = extension['extn_id'].native
            attribute_name = '_%s_value' % name
            if hasattr(self, attribute_name):
                setattr(self, attribute_name, extension['extn_value'].parsed)
            if extension['critical'].native:
                self._critical_extensions.add(name)

        self._processed_extensions = True

    @property
    def critical_extensions(self):
        """
        Returns a set of the names (or OID if not a known extension) of the
        extensions marked as critical

        :return:
            A set of unicode strings
        """

        if not self._processed_extensions:
            self._set_extensions()
        return self._critical_extensions

    @property
    def crl_value(self):
        """
        This extension is used to locate the CRL that a certificate's revocation
        is contained within.

        :return:
            None or a CrlId object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._crl_value

    @property
    def archive_cutoff_value(self):
        """
        This extension is used to indicate the date at which an archived
        (historical) certificate status entry will no longer be available.

        :return:
            None or a GeneralizedTime object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._archive_cutoff_value

    @property
    def crl_reason_value(self):
        """
        This extension indicates the reason that a certificate was revoked.

        :return:
            None or a CRLReason object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._crl_reason_value

    @property
    def invalidity_date_value(self):
        """
        This extension indicates the suspected date/time the private key was
        compromised or the certificate became invalid. This would usually be
        before the revocation date, which is when the CA processed the
        revocation.

        :return:
            None or a GeneralizedTime object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._invalidity_date_value

    @property
    def certificate_issuer_value(self):
        """
        This extension indicates the issuer of the certificate in question.

        :return:
            None or an x509.GeneralNames object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._certificate_issuer_value


class Responses(SequenceOf):
    _child_spec = SingleResponse


class ResponseDataExtensionId(ObjectIdentifier):
    _map = {
        '1.3.6.1.5.5.7.48.1.2': 'nonce',
        '1.3.6.1.5.5.7.48.1.9': 'extended_revoke',
    }


class ResponseDataExtension(Sequence):
    _fields = [
        ('extn_id', ResponseDataExtensionId),
        ('critical', Boolean, {'default': False}),
        ('extn_value', ParsableOctetString),
    ]

    _oid_pair = ('extn_id', 'extn_value')
    _oid_specs = {
        'nonce': OctetString,
        'extended_revoke': Null,
    }


class ResponseDataExtensions(SequenceOf):
    _child_spec = ResponseDataExtension


class ResponseData(Sequence):
    _fields = [
        ('version', Version, {'explicit': 0, 'default': 'v1'}),
        ('responder_id', ResponderId),
        ('produced_at', GeneralizedTime),
        ('responses', Responses),
        ('response_extensions', ResponseDataExtensions, {'explicit': 1, 'optional': True}),
    ]


class BasicOCSPResponse(Sequence):
    _fields = [
        ('tbs_response_data', ResponseData),
        ('signature_algorithm', SignedDigestAlgorithm),
        ('signature', OctetBitString),
        ('certs', Certificates, {'explicit': 0, 'optional': True}),
    ]


class ResponseBytes(Sequence):
    _fields = [
        ('response_type', ResponseType),
        ('response', ParsableOctetString),
    ]

    _oid_pair = ('response_type', 'response')
    _oid_specs = {
        'basic_ocsp_response': BasicOCSPResponse,
    }


class OCSPResponse(Sequence):
    _fields = [
        ('response_status', OCSPResponseStatus),
        ('response_bytes', ResponseBytes, {'explicit': 0, 'optional': True}),
    ]

    _processed_extensions = False
    _critical_extensions = None
    _nonce_value = None
    _extended_revoke_value = None

    def _set_extensions(self):
        """
        Sets common named extensions to private attributes and creates a list
        of critical extensions
        """

        self._critical_extensions = set()

        for extension in self['response_bytes']['response'].parsed['tbs_response_data']['response_extensions']:
            name = extension['extn_id'].native
            attribute_name = '_%s_value' % name
            if hasattr(self, attribute_name):
                setattr(self, attribute_name, extension['extn_value'].parsed)
            if extension['critical'].native:
                self._critical_extensions.add(name)

        self._processed_extensions = True

    @property
    def critical_extensions(self):
        """
        Returns a set of the names (or OID if not a known extension) of the
        extensions marked as critical

        :return:
            A set of unicode strings
        """

        if not self._processed_extensions:
            self._set_extensions()
        return self._critical_extensions

    @property
    def nonce_value(self):
        """
        This extension is used to prevent replay attacks on the request/response
        exchange

        :return:
            None or an OctetString object
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._nonce_value

    @property
    def extended_revoke_value(self):
        """
        This extension is used to signal that the responder will return a
        "revoked" status for non-issued certificates.

        :return:
            None or a Null object (if present)
        """

        if self._processed_extensions is False:
            self._set_extensions()
        return self._extended_revoke_value

    @property
    def basic_ocsp_response(self):
        """
        A shortcut into the BasicOCSPResponse sequence

        :return:
            None or an asn1crypto.ocsp.BasicOCSPResponse object
        """

        return self['response_bytes']['response'].parsed

    @property
    def response_data(self):
        """
        A shortcut into the parsed, ResponseData sequence

        :return:
            None or an asn1crypto.ocsp.ResponseData object
        """

        return self['response_bytes']['response'].parsed['tbs_response_data']



import json
import logging.config
import os

from abc import ABCMeta
from multiprocessing.pool import ThreadPool
from typing import (
    Callable,
    Any,
    List,
    Union,
    Optional,
)
from xml.etree import cElementTree as eTree

from helper import config_logger, round_half_away_from_zero
from msgbus import (
    MBEasyContext, 
    TK_SYS_NAK, 
    TK_STORECFG_GET, 
    FM_PARAM, 
    TK_HV_GLOBALRESTART,
)
from persistence import Connection, Driver, DbdException
from sysactions import (
    get_model, 
    get_podtype, 
    get_pricelist, 
    get_custom, 
    get_posot, 
    show_messagebox,
)
from systools import sys_parsefilepath


class BaseRepository(object):
    __metaclass__ = ABCMeta

    def __init__(self, mb_context):
        # type: (MBEasyContext) -> None
        self.mb_context = mb_context

    def execute_with_connection(self, method, db_name=None, service_name=None):
        # type: (Callable[[Connection], Any], int, str) -> Any
        conn = None
        try:
            conn = Driver().open(self.mb_context,
                                 dbname=str(db_name) if db_name is not None else None,
                                 service_name=service_name)
            return method(conn)
        finally:
            if conn is not None:
                conn.close()

    def execute_with_transaction(self, method, db_name=None, service_name=None):
        # type: (Callable[[Connection], Any], str, str) -> Any
        def transaction_method(conn):
            # type: (Connection) -> Any

            if conn.get_number_of_instances() != 1 and db_name is None:
                raise DbdException("Cannot open transaction without DB instance")

            conn.transaction_start()
            try:
                conn.query("BEGIN TRANSACTION")
                ret = method(conn)
                conn.query("COMMIT")
                return ret
            except Exception as _:
                conn.query("ROLLBACK")
                raise
            finally:
                conn.transaction_end()

        return self.execute_with_connection(transaction_method, db_name=db_name, service_name=service_name)

    def execute_in_all_databases(self, method, pos_list):
        # type: (Callable[[Connection], Any], List[int]) -> Any
        ret = []
        for pos_id in pos_list:
            ret.append(self._execute_in_database(method, pos_id))

        return ret

    def execute_in_all_databases_returning_flat_list(self, method, pos_list):
        # type: (Callable[[Connection], Any], List[int]) -> Any
        inner_ret = self.execute_in_all_databases(method, pos_list)

        ret = []
        for inner_list in inner_ret:
            ret.extend(inner_list)

        return ret

    def execute_parallel_in_all_databases_returning_flat_list(self, method, pos_list):
        # type: (Callable[[Connection], Any], List[int]) -> Any
        inner_ret = self.execute_parallel_in_all_databases(method, pos_list)

        ret = []
        for inner_list in inner_ret:
            ret.extend(inner_list)

        return ret

    def execute_parallel_in_all_databases(self, method, pos_list):
        # type: (Callable[[Connection], Any], List[int]) -> Any
        ret = []
        pool = ThreadPool(len(pos_list))

        for pos_id in pos_list:
            ret.append(pool.apply_async(self._execute_in_database, args=(method, pos_id)))

        pool.close()
        pool.join()

        result = [r.get() for r in ret]
        return result

    def _execute_in_database(self, method, pos_id):
        conn = None
        try:
            conn = Driver().open(self.mb_context, dbname=str(pos_id))
            return method(conn)
        finally:
            if conn is not None:
                conn.close()


def find_value_with_vars(cfg, path, default_value=None):
    value = cfg.find_value(path, default_value)
    if value is None:
        return value

    for variable_name in os.environ:
        variable_format = "{{{}}}".format(variable_name)
        if variable_format in value:
            value = value.replace(variable_format, os.environ[variable_name])
            if variable_name in ("BUNDLEDIR", "HVDATADIR"):
                value = value.replace("/./", os.sep).replace("//", os.sep).replace("/", os.sep)

    return value


def config_logger_parse_file_name(loader_path,
                                  log_name='component',
                                  configure_root=False,
                                  root_level=logging.WARN,
                                  max_files=5,
                                  max_size=5242880,
                                  add_error_handler=False):

    real_path = os.path.realpath(loader_path)
    if os.path.isfile(real_path) and real_path.endswith(".json"):
        with open(real_path, "rb") as f:
            json_loads = json.loads(f.read().decode("utf-8"))
            handlers = json_loads.get("handlers")
            if handlers:
                for handler in handlers:
                    file_name = sys_parsefilepath(handlers[handler].get("filename", ""))
                    if file_name:
                        handlers[handler]["filename"] = file_name

                loader_path = json_loads

    config_logger(loader_path, log_name, configure_root, root_level, max_files, max_size, add_error_handler)


def read_swconfig(mbcontext, key):
    rmsg = mbcontext.MB_EasySendMessage("StoreWideConfig", token=TK_STORECFG_GET, format=FM_PARAM, data=key)
    if rmsg.token == TK_SYS_NAK:
        return None

    return str(rmsg.data)


def F(value):
    if value is None:
        return 0
    return float(value)
# END F


def get_sale_line_priced_items(sale_lines, ignore_price=False):
    deleted_line_numbers = map(lambda x: x.get("lineNumber"), filter(lambda x: x.get("level") == "0" and x.get("qty") == "0", sale_lines))
    if ignore_price:
        active_sale_lines = list(filter(lambda x: (x.get("lineNumber") not in deleted_line_numbers), sale_lines))
    else:
        active_sale_lines = list(filter(lambda x: (x.get("lineNumber") not in deleted_line_numbers) and (F(x.get("itemPrice")) != 0), sale_lines))

    for sale_line in active_sale_lines:
        item_price, unit_price, item_qty, added_price, added_qty, multiplied_qty, inc_qty, level = map(sale_line.get, (("itemPrice", "unitPrice", "qty", "addedUnitPrice", "addedQty", "multipliedQty", "incQty", "level")))
        unit_price_cache = F(sale_line.get("addedUnitPrice")) or F(sale_line.get("unitPrice"))
        try:
            if F(unit_price) > 0:
                price_quantity = round_half_away_from_zero(F(item_price) / F(unit_price), 2)
                product_quantity = round_half_away_from_zero(F(item_qty) * F(multiplied_qty), 2)
                if price_quantity != product_quantity:
                    unit_price_cache = F(item_price) / product_quantity
                    unit_price = unit_price_cache
        except ZeroDivisionError:
            unit_price = 0
        try:
            correct_price = (F(unit_price) * F(item_qty) + F(added_price) * F(added_qty or item_qty)) * (F(multiplied_qty) / F(item_qty))

        except ZeroDivisionError:
            correct_price = 0

        unit_price = unit_price_cache
        if unit_price == 0 and F(item_price) > 0:
            correct_price = F(item_price)
        if not ignore_price:
            item_price = correct_price
            sale_line.set("correctItemPrice", str(item_price))

    return active_sale_lines
# END get_sale_line_priced_items


def restart_system(
        mb_context,             # type: MBEasyContext
        event_type,             # type: str
        pos_id="",              # type: str
        menu_manager_user="",   # type: str
):
    # type: (...) -> MBMessage

    _notify_global_restart(event_type, mb_context, menu_manager_user, pos_id)
    
    if pos_id != "":
        show_messagebox(
            posid=pos_id,
            message="$PLEASE_WAIT_ALL_NODES_RESTART",
        )
        
    return mb_context.MB_SendMessage(
        destination=mb_context.hv_service,
        token=TK_HV_GLOBALRESTART,
    )


def _notify_global_restart(
        event_type,         # type: str
        mb_context,         # type: MBEasyContext
        menu_manager_user,  # type: str
        pos_id              # type: str
):
    event_xml = '''<GLOBALRESTART posId="{}" userId="{}"/>'''
    data = event_xml.format(pos_id, menu_manager_user)
    mb_context.MB_EasyEvtSend(
        subject="GLOBAL_RESTART",
        type=event_type,
        xml=data,
        synchronous=True,
        timeout=-1,
    )
# END restart_system


class ModelService(object):

    def __init__(
        self,
        pos_id,  # type: Any[Union[str, int]]
    ):
        # type: (...) -> None

        self.pos_id = pos_id

    def get_price_list_from_model(
        self,
        pod_type=None,    # type: Optional[str]
        sale_type=None,   # type: Optional[str]
    ):
        # type: (...) -> str

        model = get_model(posid=self.pos_id)
        if not pod_type:
            pod_type = get_podtype(model=model)

        price_list = get_pricelist(model=model)

        is_delivery = pod_type == "DL" or sale_type == "DELIVERY"
        if is_delivery and "DL" not in price_list:
            price_list = "DL." + price_list if price_list else "DL"

        special_catalog_price_list = self._get_price_list_from_special_catalog(model=model)
        if special_catalog_price_list:
            price_list = special_catalog_price_list + "." + price_list

        if "EI" not in price_list:
            price_list += ".EI"

        return price_list

    def _get_price_list_from_special_catalog(
        self,
        model,  # type: eTree
    ):
        # type: (...) -> str

        price_list = None
        special_catalog = json.loads(self.get_special_catalog(model=model))
        price_list = self._get_table_special_catalog(
            model=model,
            price_list=price_list,
            special_catalog=special_catalog,
        )
        if not price_list and "servicePeriod" in special_catalog.keys():
            price_list = special_catalog["servicePeriod"]

        return price_list

    def _get_table_special_catalog(
        self,
        model,            # type: eTree
        price_list,       # type: str
        special_catalog,  # type: dict
    ):
        # type: (...) -> Optional[str]

        current_order = model.find("CurrentOrder/Order")
        if current_order is None:
            return price_list

        pos_ot = get_posot(model)
        order_id = current_order.get("orderId")
        order_picture = eTree.XML(pos_ot.orderPicture(orderid=order_id)).find("Order")
        if order_picture is None:
            return price_list

        order_state = order_picture.get("state")
        if order_state == "IN_PROGRESS" and self.is_table_service(order_picture=order_picture):
            order_properties = order_picture.find("CustomOrderProperties")
            table_id = order_properties.find("OrderProperty[@key='TABLE_ID']").get('value')
            if table_id is None:
                return price_list

            table = model.find("STORE_MODIFIED/Tables/Table[@id='" + table_id + "']")
            table_service_period = table.find("Properties").get("SERVICE_PERIOD_TABLE")
            if table_service_period is not None:
                price_list = json.loads(table_service_period).get("servicePeriod")
            else:
                if special_catalog.get("sectors") is not None:
                    sector = order_properties.find("OrderProperty[@key='SECTOR']").get('value')
                    if special_catalog["sectors"].get(sector) is not None:
                        price_list = special_catalog["sectors"][sector].get("servicePeriod")

        return price_list

    def get_special_catalog(
        self,
        model=None,  # type: Optional[eTree]
    ):
        # type: (...) -> str

        if model is None:
            model = get_model(posid=self.pos_id)

        special_catalog = get_custom(
            model=model,
            key='SERVICE_PERIOD',
        )
        if special_catalog in ['', None]:
            return '{}'

        return special_catalog

    @staticmethod
    def is_table_service(
        order_picture,  # type: eTree
    ):
        # type: (...) -> bool

        order_properties = order_picture.find("CustomOrderProperties")
        table_id = order_properties.find("OrderProperty[@key='TABLE_ID']") if order_properties is not None else None
        if table_id is not None:
            return True

        return False



from application.interactor import CatalogUpdaterInteractor, MediaUpdaterInteractor, UserUpdaterInteractor, \
    LoaderUpdaterInteractor, BinaryUpdaterInteractor
from application.model import BaseThread, UpdateType
from typing import Union


class Updater(object):
    def __init__(self, update_type=None, manager=None, interactor=None):
        # type: (UpdateType, BaseThread, Union[CatalogUpdaterInteractor, MediaUpdaterInteractor, UserUpdaterInteractor, LoaderUpdaterInteractor, BinaryUpdaterInteractor]) -> None # noqa

        self.update_type = update_type
        self.manager = manager
        self.interactor = interactor



from typing import Optional

from sysactions import action
from custom_sysactions import user_authorized_to_execute_action


@action
@user_authorized_to_execute_action
def clear_hold_item(
    pos_id,             # type: str
    sale_line_json,     # type: Optional[str]
):
    # type: (...) -> bool

    from table_actions import set_custom_production_tag

    return set_custom_production_tag(
        pos_id=pos_id,
        sale_line_json=sale_line_json,
    )



from _NfceCancelerResponse import NfceCancelerResponse
from _NfceCancelerResponseParserException import NfceCancelerResponseParserException
from _NfceCancelerResponseParser import NfceCancelerResponseParser
from _NfceCancelerParameters import NfceCancelerParameters
from _NfceCancelerRequestBuilder import NfceCancelerRequestBuilder
from _NfceCanceler import NfceCanceler

__all__ = ["NfceCancelerResponseParser", "NfceCancelerResponse", "NfceCanceler", "NfceCancelerParameters", "NfceCancelerRequestBuilder", "NfceCancelerResponseParserException"]


# import os
# import sys
#
# debugPath = 'C:\Program Files\JetBrains\PyCharm 2019.2\debug-eggs\pydevd-pycharm.egg'
# if os.path.exists(debugPath):
#     try:
#         sys.path.index(debugPath)
#     except:
#         sys.path.append(debugPath)
#     import pydevd


def migrate(conn, tables):
    # pydevd.settrace('localhost', port=9165, stdoutToServer=True, stderrToServer=True, suspend=True)

    update_tables_priority = [
        "Product", "ProductClassification", "ProductPart", "PriceList", "Price", "ModifierQtyLabels", "Dimensions",
        "DimensionGroups", "ProductKernelParams", "ProductCustomParams", "ProductTags", "Production", "Navigation",
        "ProductNavigation", "NavigationCustomParams", "ProductNavigationCustomParams", "Descriptions",
        "ProductDescriptions", "NavigationDescriptions", "CurrencyExchange", "TenderType", "ProductXREF", "ServicePeriod",
        "Courses", "PosSaleTypeNavigation"
    ]

    for table in update_tables_priority:
        if tables[table]["fields_OldDB"] and len(tables[table]["fields_Common"]) > 0:
            str_common_fields = ", ".join(tables[table]["fields_Common"])
            insert_str = "INSERT OR REPLACE INTO {} ({}) SELECT {} FROM old.{};"
            stmt = insert_str.format(table, str_common_fields, str_common_fields, table)
            conn.query(stmt)



from datetime import datetime

from ._OrderCondition import OrderCondition
from production.model import ProductionOrder


class DisplayTimeCondition(OrderCondition):
    def __init__(self, max_display_time):
        self.max_display_time = int(max_display_time)

    def evaluate(self, order):
        # type: (ProductionOrder) -> bool
        if order.display_time in [None, "", " "]:
            return True

        display_time = datetime.strptime(order.display_time[0:19], "%Y-%m-%dT%H:%M:%S")
        current_time = datetime.now()
        return (current_time - display_time).total_seconds() < self.max_display_time



from typing import List


class ListenedEvents(object):
    DEVICE_STATUS_CHANGED = "DEVICE_STATUS_CHANGED"
    ORDER_MODIFIED = "ORDER_MODIFIED"
    ORDER_MODIFIED_DO_TENDER = "ORDER_MODIFIED_DO_TENDER"
    ORDER_MODIFIED_DISCOUNT_APPLIED = "ORDER_MODIFIED_DISCOUNT_APPLIED"
    ORDER_MODIFIED_DISCOUNT_REMOVED = "ORDER_MODIFIED_DISCOUNT_REMOVED"
    ORDER_MODIFIED_CLEAR_TENDERS = "ORDER_MODIFIED_CLEAR_TENDERS"
    ORDER_MODIFIED_RESET_CURRENT_ORDER = "ORDER_MODIFIED_RESET_CURRENT_ORDER"


def all_events(
    order_modified_by_message,    # type: bool
):
    # type: (...) -> List[str]

    listened_events = [
        ListenedEvents.DEVICE_STATUS_CHANGED,
    ]

    if not order_modified_by_message:
        listened_events.extend([
            ListenedEvents.ORDER_MODIFIED,
            ListenedEvents.ORDER_MODIFIED_DO_TENDER,
            ListenedEvents.ORDER_MODIFIED_DISCOUNT_APPLIED,
            ListenedEvents.ORDER_MODIFIED_DISCOUNT_REMOVED,
            ListenedEvents.ORDER_MODIFIED_CLEAR_TENDERS,
            ListenedEvents.ORDER_MODIFIED_RESET_CURRENT_ORDER,
        ])

    return listened_events



# -*- coding: utf-8 -*-

import json

import sysactions
from old_helper import validar_cpf


@sysactions.action
def do_save_kiosk_customer_info(pos_id, name, cpf=''):
    model = sysactions.get_model(pos_id)

    sysactions.check_current_order(pos_id, model=model, need_order=True)

    errors = []

    pos_ot = sysactions.get_posot(model)

    if cpf and not validar_cpf(cpf):
        errors.append(['doc', '$INVALID_CPF_ERROR'])

    if errors:
        return json.dumps(errors)

    if cpf:
        pos_ot.setOrderCustomProperty('CUSTOMER_DOC', cpf)




class OrderException(Exception):
    def __init__(self, code, descr=""):
        super(OrderException, self).__init__("Error code: {0}; Error description: {1}".format(code, descr))
        self._code = int(code)
        self._descr = descr

    def get_code(self):
        return self._code

    def get_description(self):
        return self._descr

class TableServiceException(OrderException):
    def __init__(self, errmsg=""):
        super(TableServiceException, self).__init__("-1", "Table Service - {0}".format(errmsg))



from abc import ABCMeta, abstractmethod
from messagebus import Event, Message, MessageBus


class MessageHandler(object):
    """
    The methods of this class will be called whenever a new message arrives from the message bus that must be
    handled by the component
    """
    __metaclass__ = ABCMeta

    @abstractmethod
    def handle_event(self, message_bus, event):
        # type: (MessageBus, Event) -> None
        """
        This method will be called when the component receives an Event (Async or Sync). Every Sync events must be
        replied
        :param message_bus: the instance of the MessageBus that can be used to send messages to other components
        or replying the received message
        :param event: the event that must be handled by the component
        """
        raise NotImplementedError()

    def handle_message(self, message_bus, message):
        # type: (MessageBus, Message) -> None
        """
        This method will be called whenever the components received a Message. Every message must be replied.
        :param message_bus: the instance of the MessageBus that can be used to send messages to other components
        or replying the received message
        :param message: the message that must be handled bt the component
        :return:
        """
        raise NotImplementedError()



import os
from logging import getLogger

import persistence

from application.configurator import Configurator
from application.model import UpdateType
from application.repository.pos import (
    CatalogUpdaterPOSRepository,
    MediaUpdaterPOSRepository,
    UserUpdaterPOSRepository,
    LoaderUpdaterPOSRepository,
    BinaryUpdaterPOSRepository,
    VersionsPOSRepository,
    CommonPOSRepository
)
from helper import import_pydevd
from mbcontextmessagehandler import MbContextMessageBus, MbContextMessageHandler
from messagehandlerbuilder import ComponentMessageHandlerBuilder
from msgbus import MBEasyContext
from mwhelper import BaseRepository, config_logger_parse_file_name
from typing import List
from application.util import settings

LOADER_CFG = os.environ["LOADERCFG"]


def main():
    # type: () -> None

    import_pydevd(LOADER_CFG, 9145)

    configurator = Configurator()
    configurator.get_loader_configurations(LOADER_CFG)

    config_logger_parse_file_name(os.environ["LOG_PATH"], configurator.configs.service_name)
    logger = getLogger(__name__)
    logger.warning("Starting {} Component".format(configurator.configs.service_name))

    mb_context = MBEasyContext(configurator.configs.service_name)
    message_bus = MbContextMessageBus(mb_context)

    settings.init(mb_context)

    pos_repositories = _get_pos_repositories(configurator, message_bus)
    common_pos_repository = CommonPOSRepository(message_bus.mbcontext)
    message_handler_builder = ComponentMessageHandlerBuilder(configurator, pos_repositories, common_pos_repository)

    message_handler = MbContextMessageHandler(
        message_bus,
        configurator.configs.service_name,
        configurator.configs.service_name,
        configurator.configs.required_services,
        message_handler_builder
    )

    _get_store_configurations(configurator, message_bus)

    _wait_persistence(mb_context)

    message_handler.handle_events()


def _get_store_configurations(configurator, message_bus):
    configurator.get_base_url(message_bus)
    configurator.get_api_key(message_bus)

    if not configurator.configs.store_id:
        configurator.get_store_id(message_bus)


def _get_pos_repositories(configurator, message_bus):
    # type: (Configurator, MbContextMessageBus) -> List[BaseRepository]

    configs = configurator.configs
    repositories = {"GetVersions": VersionsPOSRepository(message_bus, configs)}

    if UpdateType.catalog.name in configs.updaters:
        repositories[UpdateType.catalog] = CatalogUpdaterPOSRepository(message_bus, configs)
    if UpdateType.media.name in configs.updaters:
        repositories[UpdateType.media] = MediaUpdaterPOSRepository(message_bus, configs)
    if UpdateType.user.name in configs.updaters:
        repositories[UpdateType.user] = UserUpdaterPOSRepository(message_bus, configs)
    if UpdateType.loader.name in configs.updaters:
        repositories[UpdateType.loader] = LoaderUpdaterPOSRepository(message_bus, configs)
    if UpdateType.binaries.name in configs.updaters:
        repositories[UpdateType.binaries] = BinaryUpdaterPOSRepository(message_bus, configs)

    return repositories


def _wait_persistence(
    mb_context
):
    persistence.Driver().open(mb_context).close()


if __name__ == "__main__":
    main()



from typing import Dict


class ConfiguredAlert(object):
    def __init__(
        self,
        alert_id,    # type: str
        alert_json,  # type: Dict
    ):
        # type: (...) -> None

        self.id = alert_id
        self.alert_json = alert_json



# -*- coding: utf-8 -*-

import logging
import os

from helper import config_logger, import_pydevd
from messagehandler import MessageHandler
from msgbus import MBEasyContext
import cfgtools

from rupture_event_handler import RuptureEventHandler
from rupture_events import RuptureEvents
from rupture_repository import RuptureRepository
from rupture_service import RuptureService
from services import CleanRuptureThread

LOADER_CFG = os.environ["LOADERCFG"]
REQUIRED_SERVICES = "DeliveryPersistence"
SERVICE_NAME = "Ruptura"


def main():
    import_pydevd(LOADER_CFG, 9142)

    config_logger(LOADER_CFG, SERVICE_NAME)
    logger = logging.getLogger(SERVICE_NAME)
    config = cfgtools.read(LOADER_CFG)

    mb_context = MBEasyContext(SERVICE_NAME)

    rupture_repository = RuptureRepository(mb_context, logger)
    rupture_service = RuptureService(mb_context, logger, rupture_repository, config)
    event_handler = RuptureEventHandler(mb_context, logger, rupture_service)
    message_handler = MessageHandler(mb_context, SERVICE_NAME, SERVICE_NAME, REQUIRED_SERVICES, None)

    message_handler.set_event_handler(event_handler)
    message_handler.subscribe_queue_events([
        RuptureEvents.OrderModified,
    ])
    message_handler.subscribe_non_reentrant_events([
        RuptureEvents.FullRuptureRequest,
        RuptureEvents.CleanRupture,
    ])

    if config.find_value("Rupture.CleanRuptureTimeWindow.Enabled", "false").lower() == "true":
        clean_rupture_thread = CleanRuptureThread(config, rupture_service)
        clean_rupture_thread.start()

    rupture_service.load_daily_dishes_products()
    message_handler.handle_events()


if __name__ == "__main__":
    main()



from logging import Logger

from production.box import ProductionBox
from ._OrderCondition import OrderCondition


class ConditionBox(ProductionBox):
    def __init__(self, name, condition, true_son, false_son, logger=None):
        # type: (str, OrderCondition, ProductionBox, ProductionBox, Logger) -> None
        super(ConditionBox, self).__init__(name, [true_son, false_son], logger)
        self.condition = condition
        self.true_son = true_son
        self.false_son = false_son

    def order_modified(self, order):
        if self.condition.evaluate(order):
            self.debug("Choosing: {}", self.true_son.name)
            self.true_son.order_modified(order)
        else:
            self.debug("Choosing: {}", self.false_son.name)
            self.false_son.order_modified(order)

    def __setattr__(self, name, value):
        if name == "sons" and isinstance(value, list) and len(value) >= 2:
            self.true_son = value[0]
            self.false_son = value[1]

        super(ConditionBox, self).__setattr__(name, value)



import os

import cfgtools

min_level_to_open_qsr_operator = None
min_level_to_open_ts_operator = None


def get_min_level_to_open_operator(
        user_control_type   # type: str
):
    # type: (...) -> int

    global min_level_to_open_qsr_operator, min_level_to_open_ts_operator

    if user_control_type.upper() == "QSR":
        if min_level_to_open_qsr_operator is None:
            config = cfgtools.read(os.environ["LOADERCFG"])
            min_level = config.find_value("Customizations.MinLevelOpenOperator.QSR")
            min_level_to_open_qsr_operator = int(min_level) + 1 if min_level is not None else 10

        return min_level_to_open_qsr_operator

    if user_control_type.upper() == "TS":
        if min_level_to_open_ts_operator is None:
            config = cfgtools.read(os.environ["LOADERCFG"])
            min_level = config.find_value("Customizations.MinLevelOpenOperator.TS")
            min_level_to_open_ts_operator = int(min_level) + 1 if min_level is not None else 0

        return min_level_to_open_ts_operator



from typing import Union
from xml.etree import cElementTree as eTree

from sysactions import get_user_information

from model.customexception import NotFoundUserError


def get_operator_name(
        user_id  # type: Union[int, str]
):
    # type: (...) -> Union[str, None]
    if user_id is None:
        return None

    user_xml_str = get_user_information(user_id)
    if user_xml_str is None:
        raise NotFoundUserError("USER_NOT_FOUND")

    user_xml = eTree.XML(user_xml_str)
    user_element = user_xml.find("user")

    return user_element.get("LongName")



from application.domain.ui import ScreenChanger
from sysactions import change_screen, get_model


class MsgBusScreenChanger(ScreenChanger):

    def get_main_screen(self, pos_id):
        model = get_model(pos_id)
        return model.find("Screen").get("mainId")

    def change(self, pos_id, screen_id_enum):
        change_screen(pos_id, screen_id_enum.value)

    def change_to_screen_name(self, pos_id, screen_name):
        model = get_model(pos_id)
        screen_id = model.find("Screen").get("mainId")
        change_screen(pos_id, screen_id)



import logging
from typing import List, Optional

import boto3

from messagehandler import MessageHandlerBuilder
from messageprocessor import (
    MessageProcessorMessageHandler,
    DefaultMessageProcessorExecutorFactory,
)
from topicclient.processor import ServerEventProcessor

from messageprocessorutil import LoggingProcessorCallback

logger = logging.getLogger("TopicClient")


class TopicClientMessageHandlerBuilder(MessageHandlerBuilder):
    def __init__(
        self,
        topic_arn,                  # type: str
        server_queue_url,           # type: Optional[str]
        endpoint_url,               # type: str
        region_name,                # type: str
        aws_access_key_id,          # type: str
        aws_secret_access_key,      # type: str
        subjects,                   # type: List[str]
        store_code,                 # type: str
        context,                    # type: str
        message_maximum_length,     # type: int
    ):
        # type: (...) -> None

        self.topic_arn = topic_arn
        self.server_queue_url = server_queue_url
        self.endpoint_url = endpoint_url
        self.region_name = region_name
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.subjects = subjects
        self.store_code = store_code
        self.context = context
        self.message_maximum_length = message_maximum_length

        self.message_handler = None

    def build_singletons(self):
        sqs_client = boto3.client(
            "sqs",
            aws_access_key_id=self.aws_access_key_id,
            aws_secret_access_key=self.aws_secret_access_key,
            region_name=self.region_name,
            endpoint_url=self.endpoint_url
        )

        sns_client = boto3.client(
            "sns",
            aws_access_key_id=self.aws_access_key_id,
            aws_secret_access_key=self.aws_secret_access_key,
            region_name=self.region_name,
            endpoint_url=self.endpoint_url
        )

        event_processors = {}
        for subject in self.subjects:
            server_event_processor = ServerEventProcessor(
                event_name=subject,
                store_code=self.store_code,
                context=self.context,
                sns_client=sns_client,
                sqs_client=sqs_client,
                topic_arn=self.topic_arn,
                server_queue_url=self.server_queue_url,
                message_maximum_length=self.message_maximum_length,
            )
            event_processors[subject] = server_event_processor

        callbacks = [LoggingProcessorCallback(logger)]
        self.message_handler = MessageProcessorMessageHandler(
            event_processors=event_processors,
            message_processors=None,
            executor_factory=DefaultMessageProcessorExecutorFactory(callbacks),
        )

    def build_message_handler(self, message, event):
        return self.message_handler

    def destroy_message_handler(self, message_handler):
        return

    def destroy_singletons(self):
        return



from abc import ABCMeta, abstractmethod
from logging import Logger

from application.interactor import UserUpdaterInteractor, MediaUpdaterInteractor, LoaderUpdaterInteractor, \
    CatalogUpdaterInteractor, BinaryUpdaterInteractor
from application.model import BaseThread
from application.model.configuration import Configurations
from application.model.customexception import NotInsideUpdateWindow, NotImplementedException, NoUpdateFound, \
    DisabledFunctionality
from application.repository.pos import UserUpdaterPOSRepository, MediaUpdaterPOSRepository, \
    LoaderUpdaterPOSRepository, CatalogUpdaterPOSRepository, BinaryUpdaterPOSRepository
from application.util import ValidationUtil
from typing import Union, Optional


class UpdaterManager(BaseThread):
    __metaclass__ = ABCMeta

    def __init__(self, update_name, configs, interactor, pos_repository, logger):
        # type: (str, Configurations, Union[UserUpdaterInteractor, MediaUpdaterInteractor, LoaderUpdaterInteractor, CatalogUpdaterInteractor, BinaryUpdaterInteractor], Union[UserUpdaterPOSRepository, MediaUpdaterPOSRepository, LoaderUpdaterPOSRepository, CatalogUpdaterPOSRepository, BinaryUpdaterPOSRepository], Logger) -> None # noqa

        super(UpdaterManager, self).__init__()
        self.update_name = update_name
        self.local_configs = configs.updaters[self.update_name]
        self.active_frequency = int(self.local_configs.active_frequency)
        self.interactor = interactor
        self.pos_repository = pos_repository
        self.logger = logger

    def run(self, validate_window=True):
        # type: (Optional[bool]) -> None

        if self.active_frequency == 0:
            self.logger.info("{} - Auto-update thread not active. Stopping manager".format(self.update_name))
            self.stop()
            return

        self.logger.info("{} - update thread manager started".format(self.update_name))

        self._get_update(validate_window)

    def _get_update(self, validate_window):
        # type: (bool) -> None

        disabled_update = False
        while self.running:
            try:
                self._check_update_window(validate_window)
                self.has_update_to_apply()

                self.update()
            except DisabledFunctionality as _:
                self.logger.info("{} - Auto-update thread disabled. Stopping manager".format(self.update_name))
                disabled_update = True
                self.stop()
                return
            except NotInsideUpdateWindow as _:
                self.logger.info("{} - Not inside update windows".format(self.update_name))
            except NoUpdateFound as _:
                self.logger.info("{} - No update found".format(self.update_name))
            except Exception as _:
                self.logger.exception("{} - Not expected exception on update".format(self.update_name))
            finally:
                if not disabled_update:
                    if not self.pos_repository.has_already_applied_update():
                        self.sleep(30)
                    else:
                        self.sleep(self.active_frequency)

    @abstractmethod
    def update(self):
        # type: () -> None

        raise NotImplementedException()

    def has_update_to_apply(self):
        # type: () -> None

        return

    def _check_update_window(self, validate_window):
        # type: (bool) -> None

        if not validate_window or not self.pos_repository.has_already_applied_update():
            return

        start_time = self.local_configs.window.start_time
        end_time = self.local_configs.window.end_time
        if not ValidationUtil().is_inside_allowed_window(start_time, end_time):
            str_start_time = start_time.strftime("%H:%M:%S")
            str_end_time = end_time.strftime("%H:%M:%S")
            message = "Not inside allowed window. Start: {} / End: {}".format(str_start_time, str_end_time)
            self.logger.info(message)
            raise NotInsideUpdateWindow()



from actions.events import event_publisher


def force_send_updated_model_to_ui(
    pos_id,         # type: str
    event_data="",  # type: str
):
    # type: (...) -> None

    event_subject = "POS{}".format(pos_id)
    event_publisher.publish_event(
        event_subject=event_subject,
        action_type="RESYNC_MODEL",
        event_type="RESYNC_MODEL",
        event_data=str(event_data),
    )



import json
import base64
import time

from msgbus import TK_PRN_PRINT
from messagebus import (
    DataType,
    Message,
    MessageBus,
)


class PrinterService(object):
    def __init__(
        self,
        message_bus,    # type: MessageBus
        timeout,        # type: int
    ):
        # type: (...) -> None

        self.message_bus = message_bus
        self.timeout = timeout

    def send_to_printer(
        self,               # type: PrinterService
        printer_name,       # type: str
        text,               # type: str
    ):
        #  type: (...) -> Message

        json_data = self._format_request_data(
            text=text,
        )

        message = Message(
            token=TK_PRN_PRINT,
            data_type=DataType.json,
            data=json_data,
            timeout=self.timeout,
        )

        return self.message_bus.send_message(
            component=printer_name,
            message=message,
        )

    def _format_request_data(
        self,   # type: PrinterService
        text,   # type: str
    ):
        # type: (...) -> str

        valid_until = int(time.time() + (self.timeout / 1000000))
        data = base64.b64encode(text)
        return json.dumps({"data": data, "valid_until": str(valid_until)})



from application.model.configuration import (
    DefaultManagerConfig,
    CatalogConfigurations,
    MediaConfigurations
)
from application.model.configuration import UserConfigurations
from typing import Dict, Union


class Configurations(object):
    def __init__(
            self,
            required_services=None,     # type: str
            service_name=None,          # type: str
            persistence_name=None,      # type: str
            store_id=None,              # type: str
            base_url=None,              # type: str
            api_key=None,               # type: str
            backup_directory=None,      # type: str
            updaters=None,              # type: Dict[str, Union[DefaultManagerConfig, CatalogConfigurations, MediaConfigurations, UserConfigurations, BinaryConfigurations]] # noqa
            backoffice_urls=None,       # type: str
            replace_demo_info=True,     # type: bool
            chrome_urls=None,           # type: Dict[str]
            path_to_catalog_json=None,  # type: str
        ):
        # type: (...) -> None

        self.required_services = required_services
        self.service_name = service_name
        self.persistence_name = persistence_name
        self.store_id = store_id
        self.base_url = base_url
        self.api_key = api_key
        self.backup_directory = backup_directory
        self.updaters = updaters
        self.backoffice_urls = backoffice_urls
        self.replace_demo_info = replace_demo_info
        self.chrome_urls = chrome_urls
        self.path_to_catalog_json = path_to_catalog_json



from old_helper import get_class_by_name
from pos_model import SaleItem, TaxItem
from taxcalculator.repository import NTaxCalcRepository
from typing import List, Optional
from msgbus import MBEasyContext


class GeneralTaxCalculator(object):
    def __init__(self, mb_context):
        # type: (MBEasyContext) -> None

        self.mb_context = mb_context
        self.n_tax_calc_repository = NTaxCalcRepository(mb_context)
        self.tip_taxes = None
        self.tax_calculators = []

    def calculate_all_taxes(self, sale_item, tax_calculators=None):
        # type: (SaleItem, Optional[List]) -> List[TaxItem]

        calculators = self._get_tax_calculators() if tax_calculators is None else tax_calculators
        previous_taxes = []
        for tax_calculator in calculators:
            tax_item = tax_calculator.calculate_tax(sale_item, previous_taxes)
            if tax_item is not None:
                previous_taxes.append((sale_item, tax_item))

        calculates_taxes = []
        for tax_tuple in previous_taxes:
            calculates_taxes.append(tax_tuple[1])

        return calculates_taxes

    def _get_tax_calculators(self):
        # type: () -> []

        if len(self.tax_calculators) > 0:
            return self.tax_calculators

        taxes = self.n_tax_calc_repository.load_all_taxes()

        for tax in taxes:
            tax_products = self.n_tax_calc_repository.load_all_products_from_tax(tax.code)

            klz = get_class_by_name(tax.tax_processor)
            params = tax.params
            tax_calculator = klz(self.mb_context, tax.code, tax.name, tax.fiscal_index, tax.rate, tax_products, params)

            self.tax_calculators.append(tax_calculator)

        return self.tax_calculators



# -*- coding: utf-8 -*-
import requests
import sqlite3

from datetime import datetime
from requests import Response
from typing import Dict

from application.model import UpdateType, UpdateStatus
from application.model.configuration import Configurations
from application.model.customexception import ErrorInsertingNewUpdateVersion, ErrorRestartingHV, ErrorUpdatingStep
from mbcontextmessagehandler import MbContextMessageBus
from msgbus import TK_SYS_ACK
from mwhelper import (
    BaseRepository,
    restart_system,
)
from persistence import Connection

from ._CommonPOSRepository import CommonPOSRepository


class BinaryUpdaterPOSRepository(BaseRepository):
    def __init__(self, message_bus, configs):
        # type: (MbContextMessageBus, Configurations) -> None

        super(BinaryUpdaterPOSRepository, self).__init__(message_bus.mbcontext)

        self.message_bus = message_bus
        self.configs = configs
        self.update_type = UpdateType.binaries
        self.update_value = self.update_type.value
        self.local_configs = configs.updaters[self.update_type.name]
        self.common_pos_repository = CommonPOSRepository(message_bus.mbcontext)

    def insert_new_binary_update(self, update_id, update_name):
        # type: (str, str) -> None

        try:
            def inner_func(conn):
                # type: (Connection) -> None

                date_now = str(datetime.now().isoformat())
                query = """
                INSERT OR REPLACE INTO UpdatesController
                (UpdateId, UpdateName, TypeId, StatusId, ObtainedDate)
                VALUES ('{0}', '{1}', '{2}', '{3}', '{4}')"""
                query_format = query.format(update_id, update_name, self.update_value, UpdateStatus.pending, date_now)
                conn.query(query_format)

            self.execute_with_connection(inner_func, service_name=self.configs.persistence_name)
        except Exception as ex:
            raise ErrorInsertingNewUpdateVersion(ex)

    def set_downloaded_date(self, update_id):
        self._update_step(update_id, 'DownloadedDate')

    def set_backup_date(self, update_id):
        self._update_step(update_id, 'BackupDate')

    def set_applied_date(self, update_id):
        self._update_step(update_id, 'AppliedDate')

    def set_notified_date(
        self,           # type: BinaryUpdaterPOSRepository
        update_id,      # type: str
    ):
        # type: (...) -> None

        self._update_step(
            update_id=update_id,
            update_step='NotifiedDate',
        )

    def _update_step(
        self,               # type: BinaryUpdaterPOSRepository
        update_id,          # type: int
        update_step,        # type: str
    ):
        # type: (...) -> None

        try:
            def inner_func(conn):
                # type: (Connection) -> None

                datetime_now = str(datetime.now().isoformat())
                query = """
                    UPDATE UpdatesController
                    SET {0} = '{1}'
                    WHERE 
                        Id = '{2}' AND 
                        TypeId = '{3}'
                """.format(update_step, datetime_now, update_id, self.update_value)
                conn.query(query)

            self.execute_with_connection(inner_func, service_name=self.configs.persistence_name)
        except Exception as ex:
            raise ErrorUpdatingStep(ex)

    def get_last_binary_update_id(self):
        # type: () -> int

        def inner_func(conn):
            # type: (Connection) -> int

            query = """
                SELECT UpdateId
                FROM UpdatesController
                WHERE TypeId = '{}' AND AppliedDate IS NOT NULL
                ORDER BY Id DESC
                LIMIT 1""".format(self.update_value)

            cursor = conn.select(query)
            row = cursor.get_row(0)

            if row:
                update_id = row.get_entry("UpdateId")
                return int(update_id)

            return 0

        return self.execute_with_connection(inner_func, service_name=self.configs.persistence_name)

    def get_last_binary_update_info(self):
        # type: () -> int

        def inner_func(conn):
            # type: (Connection) -> int

            query = """
                SELECT 
                    Id, 
                    NotifiedDate
                FROM UpdatesController
                WHERE 
                    TypeId = '{}' AND 
                    ObtainedDate IS NOT NULL
                ORDER BY Id DESC
                LIMIT 1
            """.format(self.update_value)

            cursor = conn.select(query)
            row = cursor.get_row(0)

            if row:
                update_id = row.get_entry("Id")
                notified_date = row.get_entry("NotifiedDate")
                return int(update_id), notified_date

            return 0, None

        return self.execute_with_connection(inner_func, service_name=self.configs.persistence_name)

    def get_last_binary_update_not_applied_controller_id(self):
        # type: () -> Dict

        def inner_func(conn):
            # type: (Connection) -> Dict

            query = """
                SELECT Id, DownloadedDate, UpdateName
                FROM UpdatesController
                WHERE TypeId = '{}' 
                AND ObtainedDate IS NOT NULL 
                AND DownloadedDate IS NOT NULL
                AND AppliedDate IS NULL
                ORDER BY Id DESC
                LIMIT 1""".format(self.update_value)

            cursor = conn.select(query)
            row = cursor.get_row(0)

            if row:
                update_id = row.get_entry("Id")
                downloaded_date = row.get_entry("DownloadedDate")
                update_name = row.get_entry("UpdateName")
                result = {
                    "update_id": update_id,
                    "downloaded_date": datetime.strptime(downloaded_date, "%Y-%m-%dT%H:%M:%S.%f"),
                    "update_name": update_name
                }
                return result

            return {}

        return self.execute_with_connection(inner_func, service_name=self.configs.persistence_name)

    def get_last_binary_update_name(self):
        # type: () -> str

        def inner_func(conn):
            # type: (Connection) -> str

            query = """
                SELECT UpdateName
                FROM UpdatesController
                WHERE TypeId = '{}' AND AppliedDate IS NOT NULL
                ORDER BY Id DESC
                LIMIT 1""".format(self.update_value)

            cursor = conn.select(query)
            row = cursor.get_row(0)

            if row:
                update_name = row.get_entry("UpdateName")
                return update_name

            return ""

        return self.execute_with_connection(inner_func, service_name=self.configs.persistence_name)

    def remove_downloaded_update(self, update_id):
        # type: (int) -> None
        try:
            def inner_func(conn):
                # type: (Connection) -> None

                query = """
                            DELETE FROM UpdatesController
                            WHERE Id = '{0}' AND TypeId = '{1}'
                        """.format(update_id, self.update_value)
                conn.query(query)

            self.execute_with_connection(inner_func, service_name=self.configs.persistence_name)
        except Exception as ex:
            raise ErrorUpdatingStep(ex)

    def has_already_applied_update(self):
        # type: () -> bool

        return self.common_pos_repository.has_already_applied_update(self.configs.persistence_name, self.update_value)

    @staticmethod
    def download_from_url(
        url     # type: str
    ):
        # type: (...) -> Response

        return requests.get(url, allow_redirects=True)

    def download_chrome_and_save(
        self,
        download_url,   # type: str
        file_path       # type: str
    ):
        # type: (...) -> bool

        response = self.download_from_url(url=download_url)
        if response.status_code == 200:
            with open(file_path, "wb") as zip_object:
                for chunk in response.iter_content(chunk_size=128):
                    zip_object.write(chunk)
            return True
        return False

    def restart_hv(self):
        # type: () -> None
        res = restart_system(
            mb_context=self.message_bus.mbcontext,
            event_type="POS_UPDATER_BINARY",
        )
        if res.token != TK_SYS_ACK:
            raise ErrorRestartingHV("Could not perform HV global restart")

    @staticmethod
    def _create_connection(db_file):
        return sqlite3.connect(db_file)



# CER encoder
from pyasn1.type import univ
from pyasn1.type import useful
from pyasn1.codec.ber import encoder
from pyasn1.compat.octets import int2oct, str2octs, null
from pyasn1 import error

class BooleanEncoder(encoder.IntegerEncoder):
    def encodeValue(self, encodeFun, client, defMode, maxChunkSize):
        if client == 0:
            substrate = int2oct(0)
        else:
            substrate = int2oct(255)
        return substrate, 0

class BitStringEncoder(encoder.BitStringEncoder):
    def encodeValue(self, encodeFun, client, defMode, maxChunkSize):
        return encoder.BitStringEncoder.encodeValue(
            self, encodeFun, client, defMode, 1000
        )

class OctetStringEncoder(encoder.OctetStringEncoder):
    def encodeValue(self, encodeFun, client, defMode, maxChunkSize):
        return encoder.OctetStringEncoder.encodeValue(
            self, encodeFun, client, defMode, 1000
        )

class RealEncoder(encoder.RealEncoder):
    def _chooseEncBase(self, value):
        m, b, e = value
        return self._dropFloatingPoint(m, b, e)

# specialized GeneralStringEncoder here

class GeneralizedTimeEncoder(OctetStringEncoder):
    zchar = str2octs('Z')
    pluschar = str2octs('+')
    minuschar = str2octs('-')
    zero = str2octs('0')
    def encodeValue(self, encodeFun, client, defMode, maxChunkSize):
        octets = client.asOctets()
# This breaks too many existing data items
#        if '.' not in octets:
#            raise error.PyAsn1Error('Format must include fraction of second: %r' % octets)
        if len(octets) < 15:
            raise error.PyAsn1Error('Bad UTC time length: %r' % octets)
        if self.pluschar in octets or self.minuschar in octets:
            raise error.PyAsn1Error('Must be UTC time: %r' % octets)
        if octets[-1] != self.zchar[0]:
            raise error.PyAsn1Error('Missing timezone specifier: %r' % octets)
        return encoder.OctetStringEncoder.encodeValue(
            self, encodeFun, client, defMode, 1000
        )

class UTCTimeEncoder(encoder.OctetStringEncoder):
    zchar = str2octs('Z')
    pluschar = str2octs('+')
    minuschar = str2octs('-')
    def encodeValue(self, encodeFun, client, defMode, maxChunkSize):
        octets = client.asOctets()
        if self.pluschar in octets or self.minuschar in octets:
            raise error.PyAsn1Error('Must be UTC time: %r' % octets)
        if octets and octets[-1] != self.zchar[0]:
            client = client.clone(octets + self.zchar)
        if len(client) != 13:
            raise error.PyAsn1Error('Bad UTC time length: %r' % client)
        return encoder.OctetStringEncoder.encodeValue(
            self, encodeFun, client, defMode, 1000
        )

class SetOfEncoder(encoder.SequenceOfEncoder):
    def encodeValue(self, encodeFun, client, defMode, maxChunkSize):
        if isinstance(client, univ.SequenceAndSetBase):
            client.setDefaultComponents()
        client.verifySizeSpec()
        substrate = null; idx = len(client)
        # This is certainly a hack but how else do I distinguish SetOf
        # from Set if they have the same tags&constraints?
        if isinstance(client, univ.SequenceAndSetBase):
            # Set
            comps = []
            while idx > 0:
                idx = idx - 1
                if client[idx] is None:  # Optional component
                    continue
                if client.getDefaultComponentByPosition(idx) == client[idx]:
                    continue
                comps.append(client[idx])
            comps.sort(key=lambda x: isinstance(x, univ.Choice) and \
                                     x.getMinTagSet() or x.getTagSet())
            for c in comps:
                substrate += encodeFun(c, defMode, maxChunkSize)
        else:
            # SetOf
            compSubs = []
            while idx > 0:
                idx = idx - 1
                compSubs.append(
                    encodeFun(client[idx], defMode, maxChunkSize)
                    )
            compSubs.sort()  # perhaps padding's not needed
            substrate = null
            for compSub in compSubs:
                substrate += compSub
        return substrate, 1

tagMap = encoder.tagMap.copy()
tagMap.update({
    univ.Boolean.tagSet: BooleanEncoder(),
    univ.BitString.tagSet: BitStringEncoder(),
    univ.OctetString.tagSet: OctetStringEncoder(),
    univ.Real.tagSet: RealEncoder(),
    useful.GeneralizedTime.tagSet: GeneralizedTimeEncoder(),
    useful.UTCTime.tagSet: UTCTimeEncoder(),
    univ.SetOf().tagSet: SetOfEncoder()  # conflcts with Set
})

typeMap = encoder.typeMap.copy()
typeMap.update({
    univ.Set.typeId: SetOfEncoder(),
    univ.SetOf.typeId: SetOfEncoder()
})

class Encoder(encoder.Encoder):
    def __call__(self, client, defMode=False, maxChunkSize=0):
        return encoder.Encoder.__call__(self, client, defMode, maxChunkSize)

encode = Encoder(tagMap, typeMap)

# EncoderFactory queries class instance and builds a map of tags -> encoders



# -*- coding: utf-8 -*-
import sysactions
import logging
import msgbus
import pyscripts
import bustoken
import os
import time

from threading import Thread
from inspect import getmembers
from xml.etree import cElementTree as eTree
from old_helper import config_logger

mbcontext = pyscripts.mbcontext
config_logger(os.environ["LOADERCFG"], 'TimeActions')
logger = logging.getLogger("TimeActions")
logger_performance = logging.getLogger("PerformanceActions")

msg_token_dict = getmembers(msgbus)
msg_token_dict.extend(getmembers(bustoken))

msg_bus_send_message = msgbus.MBEasyContext.MB_EasySendMessage
show_any_dialog = sysactions.show_any_dialog
update_any_dialog = sysactions.update_any_dialog
show_info_message = sysactions.show_info_message
set_custom = sysactions.set_custom


def get_token_name(token):
    for func_name, func_value in msg_token_dict:
        if token == func_value:
            return func_name


def _action_received(params):
    start = time.time()
    pos_id = 99
    try:
        xml, subject, evt_type, _, pos_id = params[:5]
    except Exception as _:
        xml, subject, evt_type = params[:3]

    logger.debug("POS {:2} - ### START ### - ### {} ###".format(pos_id, evt_type))

    fn = sysactions._actions.get(evt_type)
    try:
        if fn:
            args = [el.findtext('.').encode('UTF-8') for el in eTree.XML(xml).findall('Param')]
            return fn(*args)
        else:
            return None
    finally:
        message = "POS {:2} - ### ENDED ### - ### {:30} ###".format(pos_id, evt_type)
        logger.debug("{}\n".format(message))
        end = time.time()
        logger_performance.info("{} - ELAPSED: {:12.5f}".format(message, end - start))


def MB_EasySendMessage(self, dest_name, token, format=0, data=None, timeout=-1):
    token_name = get_token_name(token)
    if token == msgbus.TK_CMP_MESSAGE:
        token_name = token_name + '.{}'.format(data.split('\0')[0])

    start = None
    try:
        if token in [msgbus.TK_POS_SHOWDIALOG, msgbus.TK_POS_DIALOGRESP]:
            start = time.time()

        logger.debug("### SEND  ### - {}.{}".format(dest_name, token_name))

        if token == msgbus.TK_PRN_PRINT:
            print_thread = Thread(target=_printer_retry, args=(self, data, dest_name, format, timeout, token))
            print_thread.daemon = True
            print_thread.start()
            ret = msgbus.MBHttpMessage(msgbus.TK_SYS_ACK, None, None)
        else:
            ret = msg_bus_send_message(self, dest_name, token, format, data, timeout)
        return ret

    finally:
        logger.debug("### RECV  ### - {}.{}".format(dest_name, token_name))
        if token in [msgbus.TK_POS_SHOWDIALOG, msgbus.TK_POS_DIALOGRESP] and start is not None:
            end = time.time()
            logger_performance.info("USER Dialog Delay - ELAPSED: {:12.5f}".format(end - start))


def _printer_retry(self, data, dest_name, print_format, timeout, token):
    retry = 0
    while retry < 5:
        try:
            ret = msg_bus_send_message(self, dest_name, token, print_format, data, timeout)
            if ret.token == msgbus.TK_SYS_ACK:
                logger.info("[_printer_retry] Success printing received data")
                break
        except msgbus.MBException as _:
            logger.warn("[_printer_retry] Error printing received data", exc_info=True)

        retry += 1
        time.sleep(0.5 * retry)
        logger.warn("[_printer_retry] Retry number: {}".format(retry))
    else:
        logger.exception("[_printer_retry] printer retry failed")


def _encode_utf8(text):
    if text and isinstance(text, unicode):
        text = text.encode("utf-8")
    return text


def custom_show_any_dialog(*args, **kwargs):
    args = [_encode_utf8(x) for x in args]
    kwargs = {key: _encode_utf8(value) for key, value in kwargs.items()}
    return show_any_dialog(*args, **kwargs)


def custom_update_any_dialog(*args, **kwargs):
    args = [_encode_utf8(x) for x in args]
    kwargs = {key: _encode_utf8(value) for key, value in kwargs.items()}
    return update_any_dialog(*args, **kwargs)


def custom_show_info_message(*args, **kwargs):
    args = [_encode_utf8(x) for x in args]
    kwargs = {key: _encode_utf8(value) for key, value in kwargs.items()}
    return show_info_message(*args, **kwargs)


def custom_set_custom(*args, **kwargs):
    args = [_encode_utf8(x) for x in args]
    kwargs = {key: _encode_utf8(value) for key, value in kwargs.items()}
    return set_custom(*args, **kwargs)


def main():
    msgbus.MBEasyContext.MB_EasySendMessage = MB_EasySendMessage
    pyscripts.unsubscribe_event_listener('POS_ACTION', sysactions._action_received)
    pyscripts.subscribe_event_listener('POS_ACTION', _action_received)

    sysactions.show_any_dialog = custom_show_any_dialog
    sysactions.update_any_dialog = custom_update_any_dialog
    sysactions.show_info_message = custom_show_info_message
    sysactions.set_custom = custom_set_custom



from _TranspBuilder import TranspBuilder
from _ContextKeys import ContextKeys
from _DestinatarioBuilder import DestinatarioBuilder
from _DetListBuilder import DetListBuilder
from _DetBuilder import DetBuilder
from _DetTipBuilder import DetTipBuilder
from _EmitBuilder import EmitBuilder
from _IdeBuilder import IdeBuilder
from _InfNfeBuilder import InfNfeBuilder
from _InfNFeSuplBuilder import InfNFeSuplBuilder
from _NfceXmlPartBuilder import NfceXmlPartBuilder
from _NfeBuilder import NfeBuilder
from _PagBuilder import PagBuilder
from _TotalBuilder import TotalBuilder
from _TranspBuilder import TranspBuilder
from _NfceNamespaces import NfceNamespaces
from _TaxBuilder import TaxBuilder, TaxBuilderLucroReal, TaxBuilderSimplesNacional
from _InfRespTecBuilder import InfRespTecBuilder
from ._InfIntermedBuilder import InfIntermedBuilder
from _InfAdicBuilder import InfAdicBuilder

__all__ = [ "NfceXmlPartBuilder",
            "NfeBuilder",
            "InfNfeBuilder",
            "IdeBuilder",
            "DetBuilder",
            "DetTipBuilder",
            "EmitBuilder",
            "PagBuilder",
            "TotalBuilder",
            "TranspBuilder",
            "InfNFeSuplBuilder",
            "DestinatarioBuilder",
            "DetBuilder",
            "DetListBuilder",
            "ContextKeys",
            "NfceNamespaces",
            "TaxBuilder",
            "TaxBuilderLucroReal",
            "TaxBuilderSimplesNacional",
            "InfRespTecBuilder"
           ]



# -*- coding: utf-8 -*-

from datetime import datetime

from application.model.configuration import Configurations
from mbcontextmessagehandler import MbContextMessageBus
from mwhelper import BaseRepository


class VersionsPOSRepository(BaseRepository):
    def __init__(self, message_bus, configs):
        # type: (MbContextMessageBus, Configurations) -> None

        super(VersionsPOSRepository, self).__init__(message_bus.mbcontext)

        self.configs = configs

    def get_last_versions(self):
        # type_id: () -> Dict[int, str]

        def inner_func(conn):
            # type_id: (Connection) -> Dict[int, str]

            query = """
                        SELECT TypeId, UpdateName, AppliedDate FROM
                        (
                            SELECT *
                            FROM UpdatesController
                            WHERE AppliedDate IS NOT NULL
                            ORDER BY AppliedDate DESC
                        )
                        GROUP BY TypeId
                    """

            cursor = conn.select(query)
            if not cursor.rows():
                return {}, {}

            versions = {}
            versions_dates = {}
            for version in cursor:
                type_id = int(version.get_entry("TypeId"))
                versions[type_id] = version.get_entry("UpdateName")
                applied_date = version.get_entry("AppliedDate")[:19]
                version_date = datetime.strptime(applied_date, "%Y-%m-%dT%H:%M:%S")
                versions_dates[type_id] = datetime.strftime(version_date, "%d/%m/%Y %H:%M")

            return versions, versions_dates

        return self.execute_with_connection(inner_func, service_name=self.configs.persistence_name)



# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license

# Copyright (C) 2011,2017 Nominum, Inc.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose with or without fee is hereby granted,
# provided that the above copyright notice and this permission notice
# appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND NOMINUM DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NOMINUM BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""DNS Wire Data Helper"""

import dns.exception
from ._compat import binary_type, string_types, PY2

# Figure out what constant python passes for an unspecified slice bound.
# It's supposed to be sys.maxint, yet on 64-bit windows sys.maxint is 2^31 - 1
# but Python uses 2^63 - 1 as the constant.  Rather than making pointless
# extra comparisons, duplicating code, or weakening WireData, we just figure
# out what constant Python will use.


class _SliceUnspecifiedBound(binary_type):

    def __getitem__(self, key):
        return key.stop

    if PY2:
        def __getslice__(self, i, j):  # pylint: disable=getslice-method
            return self.__getitem__(slice(i, j))

_unspecified_bound = _SliceUnspecifiedBound()[1:]


class WireData(binary_type):
    # WireData is a binary type with stricter slicing

    def __getitem__(self, key):
        try:
            if isinstance(key, slice):
                # make sure we are not going outside of valid ranges,
                # do stricter control of boundaries than python does
                # by default
                start = key.start
                stop = key.stop

                if PY2:
                    if stop == _unspecified_bound:
                        # handle the case where the right bound is unspecified
                        stop = len(self)

                    if start < 0 or stop < 0:
                        raise dns.exception.FormError
                    # If it's not an empty slice, access left and right bounds
                    # to make sure they're valid
                    if start != stop:
                        super(WireData, self).__getitem__(start)
                        super(WireData, self).__getitem__(stop - 1)
                else:
                    for index in (start, stop):
                        if index is None:
                            continue
                        elif abs(index) > len(self):
                            raise dns.exception.FormError

                return WireData(super(WireData, self).__getitem__(
                    slice(start, stop)))
            return bytearray(self.unwrap())[key]
        except IndexError:
            raise dns.exception.FormError

    if PY2:
        def __getslice__(self, i, j):  # pylint: disable=getslice-method
            return self.__getitem__(slice(i, j))

    def __iter__(self):
        i = 0
        while 1:
            try:
                yield self[i]
                i += 1
            except dns.exception.FormError:
                raise StopIteration

    def unwrap(self):
        return binary_type(self)


def maybe_wrap(wire):
    if isinstance(wire, WireData):
        return wire
    elif isinstance(wire, binary_type):
        return WireData(wire)
    elif isinstance(wire, string_types):
        return WireData(wire.encode())
    raise ValueError("unhandled type %s" % type(wire))



class SitefParams(object):
    TRANSACTION_DATE = 105
    RECEIPT_CUSTOMER = 121
    RECEIPT_MERCHANT = 122
    TRANSACTION_PROCESSOR = 131
    CARD_TYPE = 132
    NSU = 133
    NSU_HOST = 134
    AUTH_CODE = (135, 952)
    CARD_BIN = 136
    INSTITUTION_NAME = 156
    AUTHORIZER_NAME = 158
    EXPIRATION_DATE = 513
    QR_CODE = 584
    AUTHORIZER_CNPJ = 950
    ID_MERCADO_PAGO = 4077
    DIGITAL_WALLET_ID = 106
    LAST_CARD_DIGITS = 1190
    SAT_ACQUIRER = 953
    NFCE_BRAND = 951
    MESSAGE_CODE = 2967
    MIN_SIZE = 2968
    MAX_SIZE = 2969
    TIME_OUT = 2970
    CANCEL_OPERATION_REASON = 111
